
//Refereanser til eksternt grensesnitt/kode som anvendes i denne filen
#include <iostream>
#include <fstream>
#include "AVI.h"
#include "ColorSegment.h"
#include "Feature.h"
#include "Feature_Moments.h"
#include "Feature_FD.h"
#include "Feature_Perimeter.h"
#include "Feature_Area.h"
#include "Feature_Centroid.h"
#include "Feature_BoundingBox.h"
#include "Feature_Circularity.h"
#include "Feature_Symmetry.h"
#include "Feature_Radius.h"
#include "Feature_Mean.h"
#include "NN_connection.h"


/**
 * @brief Metode som leser inn konfigurasjonsdata fra fil.
 *
 * @param *filename Peker til navnet p systemets konfigurasjonsfil.
 */
void AVI::readConfiguration(const char *fileName)
{
	ifstream in(fileName);
	string line;
	vector<string> lines;

	if (!in)
	{
		cout << "Konfigurasjonsfilen kan ikke aksesseres!" << endl;
		exit(0);
	}

	while (!in.eof())
	{
		getline(in,line);
		if ( line[0] == '=' ) lines.push_back( line.erase(0,1) );
	} 

	in.close();

	const char *folderNameApproved, *folderNameRejected;

	string *featureMoment = new string[7];
	string *featureFD = new string[6];
	string *featurePerimeter = new string[1];
	string *featureArea = new string[1];
	string *featureCentroid = new string[2];
	string *featureBoundingBox = new string[4];
	string *featureCircularity = new string[1];
	string *featureSymmetry = new string[1];	
	string *featureRadius = new string[3];
	string *featureMean = new string[3];

	for (unsigned int i=0;i<lines.size();i++)
	{
		switch(i)
		{
			case 0: weightArea = (float) atof( lines[i].c_str() ); break;
			case 1: weightCentroid = (float) atof( lines[i].c_str() ); break;
			case 2: weightBoundingBox = (float) atof( lines[i].c_str() ); break;
			case 3: windowPartitioning = atoi( lines[i].c_str() ); break;
			case 4: minIntensity = atoi( lines[i].c_str() ); break;
			case 5: maxInhomogenity = atoi( lines[i].c_str() ); break;
			case 6: maxBlurIterations = atoi( lines[i].c_str() ); break;
			case 7: minArea = atoi( lines[i].c_str() ); break;
			case 8: grow = atoi( lines[i].c_str() ); break;
			case 9: writeSegmentationToFile = atoi( lines[i].c_str() ); break;
			case 10: folderNameApproved = lines[i].c_str(); break;
			case 11: folderNameRejected = lines[i].c_str(); break;
			case 12: folderNameOnline = lines[i].c_str(); break;
			case 13: onlineResultFileName = lines[i].c_str(); break;
			case 14: trainingDataFileName = lines[i].c_str(); break;
			case 15: executeDataFileName = lines[i].c_str(); break;
			case 16: selectedFeaturesFileName = lines[i].c_str(); break;
			case 17: nnNumNeuronsHidden = atoi( lines[i].c_str() ); break;
			case 18: nnDesiredError = (float) atof( lines[i].c_str() ); break;
			case 19: nnMaxEpochs = atoi( lines[i].c_str() ); break;
			case 20: nnEpochsBetweenReports = atoi( lines[i].c_str() ); break;
			case 21: featureMoment[0] = lines[i].c_str(); break;
			case 22: featureMoment[1] = lines[i].c_str(); break;			
			case 23: featureMoment[2] = lines[i].c_str(); break;
			case 24: featureMoment[3] = lines[i].c_str(); break;
			case 25: featureMoment[4] = lines[i].c_str(); break;
			case 26: featureMoment[5] = lines[i].c_str(); break;
			case 27: featureMoment[6] = lines[i].c_str(); break;
			case 28: featureFD[0] = lines[i].c_str(); break;
			case 29: featureFD[1] = lines[i].c_str(); break;
			case 30: featurePerimeter[0] = lines[i].c_str(); break;
			case 31: featureArea[0] = lines[i].c_str(); break;
			case 32: featureCentroid[0] = lines[i].c_str(); break;
			case 33: featureCentroid[1] = lines[i].c_str(); break;
			case 34: featureBoundingBox[0] = lines[i].c_str(); break;
			case 35: featureBoundingBox[1] = lines[i].c_str(); break;
			case 36: featureBoundingBox[2] = lines[i].c_str(); break;
			case 37: featureBoundingBox[3] = lines[i].c_str(); break;
			case 38: featureCircularity[0] = lines[i].c_str(); break;
			case 39: featureSymmetry[0] = lines[i].c_str(); break;
			case 40: featureRadius[0] = lines[i].c_str(); break;
			case 41: featureRadius[1] = lines[i].c_str(); break;
			case 42: featureRadius[2] = lines[i].c_str(); break;
			case 43: featureMean[0] = lines[i].c_str(); break;
			case 44: featureMean[1] = lines[i].c_str(); break;
			case 45: featureMean[2] = lines[i].c_str(); break;
		}
	}

	numberOfFeatureCandidates = 0;

	//Statistical Moments
	Feature_Moments *moments = new Feature_Moments();
	initializeFeature(moments, featureMoment);

	//Fourier Descriptors
	Feature_FD *fd = new Feature_FD();
	initializeFeature(fd, featureFD);

	//Perimeter
	Feature_Perimeter *perimeter = new Feature_Perimeter();
	initializeFeature(perimeter, featurePerimeter);

	//Area
	Feature_Area *area = new Feature_Area();
	initializeFeature(area, featureArea);

	//Centroid
	Feature_Centroid *centroid = new Feature_Centroid();
	initializeFeature(centroid, featureCentroid);

	//Bounding box
	Feature_BoundingBox *bb = new Feature_BoundingBox();
	initializeFeature(bb, featureBoundingBox);

	//Circularity
	Feature_Circularity *circularity = new Feature_Circularity();
	initializeFeature(circularity, featureCircularity);

	//Radius
	Feature_Radius *radius = new Feature_Radius();
	initializeFeature(radius, featureRadius);

	//Symmetry
	Feature_Symmetry *symmetry = new Feature_Symmetry();
	initializeFeature(symmetry, featureSymmetry);

	//Mean color
	Feature_Mean *mean = new Feature_Mean();
	initializeFeature(mean, featureMean);

	//Flere egenskapskandidater
	//....

	appString = ".\\";
	appString.append(folderNameApproved);
	appString += "\\";

	rejString = ".\\";
	rejString.append(folderNameRejected);
	rejString += "\\";

	onlineString = ".\\";
	onlineString.append(folderNameOnline);
	onlineString += "\\";

	using namespace System;
	using namespace System::IO;

	approvedNames = new vector<string>;
	rejectedNames = new vector<string>;
	onlineNames = new vector<string>;

	//De flgende kodelinjer for  lese filer i en katalog er kopiert fra forum p http://www.msdn.com
	DirectoryInfo* di = new DirectoryInfo( appString.c_str() );
    FileInfo* fi[] = di->GetFiles();
    Collections::IEnumerator* myEnum = fi->GetEnumerator();
    while (myEnum->MoveNext()) {
        FileInfo* fiTemp = __try_cast<FileInfo*>(myEnum->Current);
		approvedNames->push_back( appString + ToCppString(fiTemp->Name) );
    }

	di = new DirectoryInfo( rejString.c_str() );
    fi = di->GetFiles();
    myEnum = fi->GetEnumerator();
    while (myEnum->MoveNext()) {
        FileInfo* fiTemp = __try_cast<FileInfo*>(myEnum->Current);
		rejectedNames->push_back( rejString + ToCppString(fiTemp->Name) );
    }

	di = new DirectoryInfo( onlineString.c_str() );
    fi = di->GetFiles();
    myEnum = fi->GetEnumerator();
    while (myEnum->MoveNext()) {
        FileInfo* fiTemp = __try_cast<FileInfo*>(myEnum->Current);
		onlineNames->push_back( onlineString + ToCppString(fiTemp->Name) );
    }
}



/**
 * @brief Leser en linje fra konfigureringsfilen
 *
 * @param input Streng som representerer en linje (med to hovedkraven) i konfigureringsfilen.
 *
 * @return En vektor med to verdier som tilsvarer de to hovedkravene for aktuell egenskapskandidat.
 */
float * AVI::readReq(string input)
{
	float *f = new float[2];
	for (unsigned int i=0;i<input.length();i++)
	{
		if ( input.at(i) == ' ' )
		{
			f[0] = (float) atof( input.substr(0,i).c_str() );
			f[1] = (float) atof( input.substr(i+1, input.length()-i ).c_str() );
			return f;
		}
	}
	return f;
}



/**
 * @brief Initialiserer egenskapskandidater
 *
 * @param feature Peker til egenskapskandidatobjektet.
 * @param lines Peker til egenskapskandidatens hovedkrav 1 og 2 fra konfigurasjonsfilen.
 */
void AVI::initializeFeature(Feature *feature, string *lines)
{
	float **reqs = new float*[feature->getDim()]; 
	for (int i=0;i<feature->getDim();i++)
	{
		reqs[i] = readReq( lines[i] );
	}
	feature->initialize(reqs);
	featureCandidates.push_back( feature );
	numberOfFeatureCandidates++;
}



/**
 * @brief Konverterer fra System::String til string
 *
 * Denne metoden er kopiert fra MSCE Software Forums, http://www.mcse.ms
 *
 * @param str Peker til et System::String-objekt
 *	
 * @return Inputverdi p string-objekt form.
 */
string AVI::ToCppString(System::String *str)
{
	if (str == 0)
	{
		return(string());
	}
	System::IntPtr
	ptr(System::Runtime::InteropServices::Marshal::StringToHGlobalAnsi(str));
	std::string ret(static_cast<const char *>(static_cast<void *>(ptr)));
	System::Runtime::InteropServices::Marshal::FreeCoTaskMem(ptr);
	return(ret);
}



/**
 * @brief Deler inn bildet i vinduer
 *
 * @param dimension Peker til vektor som holder bildets x og y-dimensjon.
 */
void AVI::windowAssignment(int *dimension)
{
	numberOfWindows = windowPartitioning * windowPartitioning;
	windowVectorMapX = new unsigned int[windowPartitioning-1];
	windowVectorMapY = new unsigned int[windowPartitioning-1];
	float factor = 1 /  (float) windowPartitioning;
	int deltaX = (int) ( dimension[0] * factor );
	int deltaY = (int) ( dimension[1] * factor );
	int posX=0, posY=0;
	for (int i=0;i<windowPartitioning-1;i++)
	{
		posX += deltaX;
		posY += deltaY;
		windowVectorMapX[i] = posX;
		windowVectorMapY[i] = posY;
	}
}



/**
 * @brief Henter aktuelt vindu i bildet.
 *
 * @param p Pikselobjekt.
 * 
 * @return Det vinduet som den aktuelle pikselen befinner seg i.
 */
int AVI::getWindow(Pixel p)
{
	int indexX=0, indexY=0;
	while ( p.x > windowVectorMapX[indexX] )
	{
		indexX++;
		if (indexX == windowPartitioning-1) break;
	}
	while ( p.y > windowVectorMapY[indexY] )
	{
		indexY++;
		if (indexY == windowPartitioning-1) break;
	}
	return indexY * windowPartitioning + indexX;
}



/**
 * @brief Kalkulerer den lille egenskapsmengden p et bilde.
 *
 * @param image Peker til et bildeobjekt.
 */
void AVI::calculateRegionData(Image *image)
{
	std::vector<Region*> *regions = image->getRegions();
	for (unsigned int i=0;i<regions->size();i++)
	{
		regions->at(i)->calculateCentroid();
		regions->at(i)->calculateBoundingBox();
		regions->at(i)->setWindow( getWindow( regions->at(i)->getCentroid() ) );
	}
}



/**
 * @brief Lager objekter av bildepar (Image_pair-objekter).
 *
 * @return Antall bildeparobjekter.
 */
int AVI::createImagePairs()
{
	sort = new Sort();

	int sum=0, c=0;
	for (unsigned int i=1;i<approvedImages.size();i++) { sum += i; }

	float * weight = new float[3];
	weight[0] = weightArea;
	weight[1] = weightCentroid;
	weight[2] = weightBoundingBox;

	int numberOfPairs = sum + approvedImages.size() * rejectedImages.size();
	imagePairs = new Image_pair[ numberOfPairs ];

	//Lage bildepar av godkjente bilder
	for (unsigned int i=0;i<approvedImages.size();i++)
	{
		for (unsigned int j=i+1;j<approvedImages.size();j++)
		{
			Image *im1 = approvedImages.at(i);
			Image *im2 = approvedImages.at(j);
			if ( im1->getRegions()->size() <= im2->getRegions()->size() )
			{
				imagePairs[c] = * new Image_pair( im1, im2, true, weight, sort );
			}
			else
			{
				imagePairs[c] = * new Image_pair( im2, im1, true, weight, sort );
			}
			c++;
		}
	}

	//Lage bildepar av godkjent og underkjent bilde
	for (unsigned int i=0;i<approvedImages.size();i++)
	{
		for (unsigned int j=0;j<rejectedImages.size();j++)
		{
			Image *imA = approvedImages.at(i);
			Image *imR = rejectedImages.at(j);
			if ( imA->getRegions()->size() <= imR->getRegions()->size() )
			{
				imagePairs[c] = * new Image_pair( imA, imR, false, weight, sort );
			}
			else
			{
				imagePairs[c] = * new Image_pair( imR, imA, false, weight, sort );
			}
			c++;
		}
	}

	return numberOfPairs;
}



/**
 * @brief Beregner den store egenskapskandidatmengden
 *
 * @param numberOfPairs Antall bildeparobjekter.
 */
void AVI::calculateBigFeatureSet(int numberOfPairs)
{
	for (int i=0;i<numberOfPairs;i++)
	{
		int *map = imagePairs[i].getBestCorrespondenceVector();
		int smallLength = imagePairs[i].getSmallImage()->getRegions()->size();
		int bigLength = imagePairs[i].getBigImage()->getRegions()->size();
		for (int j=0;j<smallLength*bigLength;j++)
		{
			if ( map[j]==0 ) continue;
            int w = j % bigLength;
			int h = j / bigLength;
			Region *r1 = imagePairs[i].getSmallImage()->getRegions()->at(h);
			Region *r2 = imagePairs[i].getBigImage()->getRegions()->at(w);			
			if ( !r1->getCalculated() ) calculateRegionFeatureVector(r1);
			if ( !r2->getCalculated() ) calculateRegionFeatureVector(r2);
		}
	}
}



/**
 * @brief Beregner egenskapskandidatverdier for en aktuell region.
 *
 * @param region Peker til aktuell region.
 */
void AVI::calculateRegionFeatureVector(Region *region)
{
	region->setFeatureVectorRowSize(numberOfFeatureCandidates);
	region->setCalculated();
	for (int i=0;i<numberOfFeatureCandidates;i++)
			region->setFeatureVectorValue( i, featureCandidates[i]->getDim(), featureCandidates[i]->calculate(region) );
	cout << ".";
}



/**
 * @brief Beregner egenskapskandidatenes nkkeldata.
 *
 * Beregner maskimal og gjennomsnittlig forskjell mellom godkjente bilders objekter seg imellom,
 * og gjennomsnittlige forskjell mellom godkjente og underkjente bilders objekter.
 *
 * @param numberOfPairs Antall bildeparobjekter.
 */
void AVI::calculateFeatureCandidatesKeyData(int numberOfPairs)
{
	for (int i=0;i<numberOfFeatureCandidates;i++)
	{
		int numberOfHomogen=0, numberOfInhomogen=0;
		for (int j=0;j<numberOfPairs;j++)
		{
			int *map = imagePairs[j].getBestCorrespondenceVector();
			int smallLength = imagePairs[j].getSmallImage()->getRegions()->size();
			int bigLength = imagePairs[j].getBigImage()->getRegions()->size();
			for (int k=0;k<smallLength*bigLength;k++)
			{
				if ( map[k]==0 ) continue;
	            int w = k % bigLength;
				int h = k / bigLength;
				Region *r1 = imagePairs[j].getSmallImage()->getRegions()->at(h);
				Region *r2 = imagePairs[j].getBigImage()->getRegions()->at(w);
				float *diff = featureCandidates[i]->difference(r1,r2);
				if ( imagePairs[j].getHomogenious() )
				{
					numberOfHomogen++;
					featureCandidates[i]->add( true, diff );
					featureCandidates[i]->maxHomogen( diff );
				}
				else
				{
					numberOfInhomogen++;
					featureCandidates[i]->add( false, diff );
				}
			}
		}
		if ( numberOfHomogen > 0) featureCandidates[i]->divide( true, numberOfHomogen );
		if ( numberOfInhomogen > 0) featureCandidates[i]->divide( false, numberOfInhomogen );
	}
}



/**
 * @brief Henter de egenskapskandidatene som er egnet for bildesettet.
 *
 * @return Peker til dynamisk vektor som holder egnede egenskapskandidater
 */
vector<int> * AVI::selectCandidateFeatures()
{
	ofstream out( selectedFeaturesFileName.c_str() );
	vector<int> *selectedFeatures = new vector<int>;
	for (int i=0; i<numberOfFeatureCandidates; i++)
	{
		if ( featureCandidates[i]->getCredit() > 0 ) 
		{
			out << featureCandidates[i]->getId() << " " << featureCandidates[i]->getName() << endl;
			selectedFeatures->push_back(i);
		}
	}
	out.close();
	return selectedFeatures;
}



/**
 * @brief Beregner egnede egenskapskandidater for regioner som ikke er mappet i bildeparene.
 *
 * Dette gjres for at alle segmenterte regioner i bildet skal vre med  bidra til 
 * inputvektoren til det nevral nettverket, som ved "online" eksekvering, og ikke bare de 
 * regionene som er mappet i bildeparene.
 *
 * @param selectedFeatures Peker til dynamisk vektor som holder de egnede egenskapskandidatene.
 */
void AVI::calculateLeftovers(vector<int> *selectedFeatures)
{
	for (unsigned int i=0;i<approvedImages.size();i++)
	{
		Image *image = approvedImages.at(i);
		for (unsigned int j=0;j<image->getRegions()->size();j++)
		{
			Region *region = image->getRegions()->at(j);
			if ( !region->getCalculated() ) calculateRegionFeatureVector(region);
		}
	}
	for (unsigned int i=0;i<rejectedImages.size();i++)
	{
		Image *image = rejectedImages.at(i);
		for (unsigned int j=0;j<image->getRegions()->size();j++)
		{
			Region *region = image->getRegions()->at(j);
			if ( !region->getCalculated() ) calculateRegionFeatureVector(region);
		}
	}
}



/**
 * @brief Skriver treningsdata til fil.
 *
 * @param selectedFeatures Peker til dynamisk vektor som holder de egnede egenskapskandidatene.
 * @param filename Peker til navnet til fil med treningsdata.
 *
 * @return Strrelse til det nevralet netverkets inputvektor.
 */
int AVI::writeTrainingDataToFile(vector<int> *selectedFeatures, const char *filename)
{

	if ( selectedFeatures->size() == 0 ) 
	{
		cout << "Ingen kandidategenskaper innfrir kravene i konfigurasjonsfilen!" << endl;
		exit(0);
	}

	FILE *out = fopen(filename,"w");
	dimSum = 0;
	for (unsigned int i=0;i<selectedFeatures->size();i++)
	{
		dimSum += featureCandidates[ selectedFeatures->at(i) ]->getDim();
	}
	unsigned int nnInputVectorSize = numberOfWindows * dimSum;
	float *nnInputVector = new float[nnInputVectorSize];
	for (int i=0; i<nnInputVectorSize; i++)
	{
		nnInputVector[i] = 0;
	}

	fprintf( out, "%d", ( approvedImages.size()+rejectedImages.size() ) );
	fprintf( out, " %d", nnInputVectorSize );
	fprintf( out, " %d\n", 1 );
	
	for (unsigned int i=0;i<approvedImages.size();i++)
	{
		Image *aImage = approvedImages.at(i);
		for (unsigned int j=0;j<aImage->getRegions()->size();j++)
		{
			Region *aRegion = aImage->getRegions()->at(j);
			int pos = aRegion->getWindow() * dimSum;
			insertValuesIntoVector(pos, nnInputVector, aRegion, selectedFeatures);
		}
		writeTrainingVector(nnInputVectorSize, nnInputVector, out);
		fprintf( out, "\n1\n");
	}

	for (unsigned int i=0;i<rejectedImages.size();i++)
	{
		Image *rImage = rejectedImages.at(i);
		for (unsigned int j=0;j<rImage->getRegions()->size();j++)
		{
			Region *rRegion = rImage->getRegions()->at(j);
			int pos = rRegion->getWindow() * dimSum;
			insertValuesIntoVector(pos, nnInputVector, rRegion, selectedFeatures);
		}
		writeTrainingVector(nnInputVectorSize, nnInputVector, out);
		fprintf( out, "\n-1\n");
	}
	fclose(out);

	return nnInputVectorSize;
}



/**
 * @brief Skriver treningsdata til fil. 
 *
 * Metode som brukes av writeTrainingDataToFile(...)-metoden.
 *
 * @param nnInputVectorSize Strrelse p inputvektor til det nevrale nettverket.
 * @param nnInputVector Peker til vektor som holder beregnede egenskapskandidatverdier.
 * @param out Peker til filobjekt hvor utskrift pgr.
 */
void AVI::writeTrainingVector(unsigned int nnInputVectorSize, float *nnInputVector, FILE *out)
{
	for (unsigned int i=0;i<nnInputVectorSize;i++) 
	{	
		fprintf(out, "%10.2f", roundOf(nnInputVector[i]) );
		fprintf(out, " ");
	}
}



/**
 * @brief Oppdaterer inputvektor for det nevrale nettverket med beregnede egenskapskandidatverdier.
 *
 * @param start Indeks som angir aktuell startposisjon i nnInputVector.
 * @param nnInputVector Peker til inputvektor for det nevrale nettverket.
 * @param region Peker til aktuell region som brukes til  finne beregnede verdier for egenskapskandidater.
 * @param selectedFeatures Peker til dynamisk vektor som holder de egnede egenskapskandidatene.
 */
void AVI::insertValuesIntoVector(int start, float *nnInputVector, Region *region, vector<int> *selectedFeatures)
{
	for (unsigned int i=0;i<selectedFeatures->size();i++)
	{
		for (int j=0;j<featureCandidates[selectedFeatures->at(i)]->getDim();j++)
		{
			nnInputVector[start] = region->getFeatureValue( selectedFeatures->at(i), j);
			start++;
		}
	}
}



/**
 * @brief Henter ut en liste av egnede egenskapskandidater fra fil.
 *
 * @param fileName Peker til navn p fil som angir hvilke egenskapskandidater som er egnede.
 *	
 * @return Peker til en dynamisk vektor med pekere til egnede egenskapskandidatobjekter.
 */
vector<Feature*> * AVI::readSelectedFeaturesFromFile(const char *fileName)
{
	ifstream in(fileName);
	string line;
	vector<Feature*> *selectedFeatures = new vector<Feature*>;

	if (!in)
	{
		cout << fileName << " kan ikke aksesseres!" << endl;
		exit(0);
	}

	dimSum = 0;

	while (!in.eof())
	{
		getline(in,line);
		if ( line!="" )
		{
			int featureID = atoi( line.substr(0, line.find_first_of(' ')).c_str() );
			Feature *f = getFeatureFromID(featureID);
			selectedFeatures->push_back( f );
			dimSum += f->getDim();
		}
	} 

	in.close();

	return selectedFeatures;
}



/**
 * @brief Henter ut det aktuelle egenskapskandidatobjektet tilsvarende identifikasjon gitt i fil.
 *
 * @param id Identifikasjon p aktuell egenskapskandidat fra fil.
 *	
 * @return Peker til det aktuelle egenskapskandidatobjektet.
 */
Feature * AVI::getFeatureFromID(int id)
{
	Feature *feature;
	switch(id)
	{
		case 0: feature = new Feature_Moments(); break;
		case 1: feature = new Feature_FD(); break;
		case 2: feature = new Feature_Perimeter(); break;
		case 3: feature = new Feature_Area(); break;
		case 4: feature = new Feature_Centroid(); break;
		case 5: feature = new Feature_BoundingBox(); break;
		case 6: feature = new Feature_Circularity(); break;
		case 7: feature = new Feature_Radius(); break;
		case 8: feature = new Feature_Symmetry(); break;
		case 9: feature = new Feature_Mean(); break;
		
		//Flere egenskapskandidater
		//....

	}
	return feature;
}



/**
 * @brief Beregner inputvektor til det nevrale nettverket i eksekveringsfase.
 *
 * @param selectedFeatures Peker til dynamisk vektor som holder de egnede egenskapskandidatene.
 * @param regions Peker til en dynamisk vektor som holder pekere til regionobjekter.
 *	
 * @return Peker til vektor som holder inputvektor til det nevrale nettverket.
 */
float * AVI::calculateOnlineImageData(vector<Feature*> *selectedFeatures, vector<Region*> *regions)
{
	int nnInputVectorSize = numberOfWindows * dimSum;
	float *nnInputVector = new float[nnInputVectorSize];
	for (int i=0; i<nnInputVectorSize; i++)
	{
		nnInputVector[i] = 0;
	}

	for (unsigned int i=0;i<regions->size();i++)
	{
		Region *r = regions->at(i);
		r->setFeatureVectorRowSize( selectedFeatures->size() );
		int pos = r->getWindow() * dimSum;
		for (unsigned int j=0;j<selectedFeatures->size();j++)
		{
			r->setFeatureVectorValue( j, selectedFeatures->at(j)->getDim(), selectedFeatures->at(j)->calculate(r) );
			for (int k=0;k<selectedFeatures->at(j)->getDim();k++)
			{
				nnInputVector[pos] = r->getFeatureValue( j, k );
				pos++;			
			}
		}
	}
	return nnInputVector;
}



/**
 * @brief Avrunder flyttall med nedre og vre grense.
 *
 * @param x Verdi som skal avrundes.
 *	
 * @return Avrundet verdi av flyttallet.
 */
float AVI::roundOf(float x)
{
	if ( x < -NN_INFINITY ) return -NN_INFINITY;
	if ( x > NN_INFINITY ) return NN_INFINITY;
	return x;
}



/**
 * @brief Konstruktor
 *
 * @param online Parameter som angir om systemet skal kjre i offline eller online (med verdier henholdsvis 0 og 1).
 * @param configurationFileName Peker til navnet p systemets konfigurasjonsfil.
 */
AVI::AVI(int online, const char *configurationFileName)
{
	readConfiguration(configurationFileName);
	NN_connection nn;
	ColorSegment cs(minIntensity, maxInhomogenity, maxBlurIterations, minArea, grow, writeSegmentationToFile);
	if ( online == 0 )
	{
		if ( approvedNames->size() > 0 )
		{
			//Antar samme dimensjon for alle bilder
			windowAssignment( cs.getDimension( approvedNames->at(0).c_str() ) );

			for (unsigned int i=0;i<approvedNames->size();i++)
			{
				Image *image = cs.segment( approvedNames->at(i).c_str(), appString.length() );
				calculateRegionData(image);
				approvedImages.push_back(image);
				cout << "Segmentering ferdig av godkjent bilde: " << image->getName() << endl;
			}
			for (unsigned int i=0;i<rejectedNames->size();i++)
			{
				Image *image = cs.segment( rejectedNames->at(i).c_str(), rejString.length() );
				calculateRegionData(image);
				rejectedImages.push_back(image);
				cout << "Segmentering ferdig av underkjent bilde: " << image->getName() << endl;
			}
			
			int numberOfPairs = createImagePairs();
			cout << "Beregner egenskapskandidater";
			calculateBigFeatureSet(numberOfPairs);
			cout << endl;
			calculateFeatureCandidatesKeyData(numberOfPairs);

			vector<int> *selectedFeatures = selectCandidateFeatures();
			calculateLeftovers(selectedFeatures);
			int numInput = writeTrainingDataToFile(selectedFeatures, trainingDataFileName.c_str());

			nn.createNetwork(numInput, nnNumNeuronsHidden, 1);

			cout << "Trener nevralt nettverk" << endl;
			nn.train( trainingDataFileName.c_str(), nnMaxEpochs, nnEpochsBetweenReports, 
					nnDesiredError, executeDataFileName.c_str() );

		}
		else
		{
			cout << "Ingen bilder er tilgjengelige for trening i \"" << appString << "\""
				 << " og/eller " << "\"" << rejString << "\"" << endl;
		}
	}
	else
	{
		if ( onlineNames->size() == 0 )
		{
			cout << "Ingen bilder er tilgjengelige for online eksekvering i \"" << onlineString << "\"" << endl;
		}
		else
		{
			//Antar samme dimensjon for alle bilder
			windowAssignment( cs.getDimension( onlineNames->at(0).c_str() ) );
			nn.createNetworkFromFile( executeDataFileName.c_str() );
			vector<Feature*> *selectedFeatures = readSelectedFeaturesFromFile( selectedFeaturesFileName.c_str() );
			ofstream out( onlineResultFileName.c_str() );
			for (unsigned int i=0;i<onlineNames->size();i++)
			{
				Image *image = cs.segment( onlineNames->at(i).c_str(), onlineString.length() );
				cout << "Online segmentering ferdig av bilde: " << image->getName() << endl;
				calculateRegionData(image);
				float *nnInputVector = calculateOnlineImageData( selectedFeatures, image->getRegions() );
				int approved = nn.execute( nnInputVector );
				out << image->getName() << " " << approved << endl;
				cout << "	Resultat (0:underkjent, 1:godkjent) = " << approved << endl;
			}		
			out.close();
			nn.destroy();
		}
	}
}



/**
 * @brief Destruktor
 */
AVI::~AVI()
{
	//Tom destruktor
}



/**
 * @brief Trd som kjrer AVI-systemet
 */
int main(int argc, char* argv[])
{
	if ( argc < 3 )
	{
		cout << "Applikasjonen krever parametrene: <konfigurasjonsfil> <0=offline/1=online>" << endl;
	}
	else
	{
		AVI s( atoi(argv[2]), argv[1] );
	}
	return 0;
}