#pragma once

//Refereanser til ekstern kode som anvendes i denne filen
#include "Image.h"

/**
 * @class ColorSegment
 * @brief Klasse som utfrer fargesegmentering p bilder
 * 
 * Denne klassen er en mulig implementasjon av Richard Blake's fargesegmenteringsmetode. Teorien
 * er gitt i kurset TDT4265 Datasyn ved NTNU, http://dionysus.idi.ntnu.no/~cv/COLSSEGA.html og 
 * alternativt http://rover.idi.ntnu.no/~cv/COLSSEGAA.shtml. Implementasjonen skiller seg fra 
 * det andre steget i denne teorien, ved  anvende region growing basert p gjennomsnittlige 
 * fargeverdier i regioner, istedet for bruk av kromatisitetsdiagram. Klassen lar strre regioner
 * vokse ved  "spise" opp piksler fra regioner med et areal som er under en gitt terskelverdi. 
 * "Spisingen" skjer ved  flge konturen til disse sm regionene, og tildele konturpikslene til 
 * en strre naboregion, som har strst likhet i gjennomsnittlig fargeverdi.
 *
 * Denne koden gjr bruk av "Image Processing Library 98 (IPL98) versjon 2.20", for  lese og 
 * skrive bildeformater. Mer informasjon om dette bilioteket finnes p IPL98 sine websider, 
 * http://www.mip.sdu.dk/ipl98.
 *
 * @version 1.0
 * @author Per Gunnar Brdsen
 */
class ColorSegment
{
	///En terskel som angir et minstekrav til fargeintensitet. Piksler med intensiteter (r*r+g*g+b*b) mindre enn denne terskelen vil ikke bidra i segmenteringsprosessen.
	int minIntensity;;

	///En terskel som angir maksimalt tillatte inhomogenitet i regioner. Dvs at pikslene P1 og P2 bare sles sammen til en region dersom |r1-r2| + |g1-g2| + |b1-b2| <= maxInhomogenity, der r1 og r2 er den rde intensiteten til henholdsvis piksel P1 og P2 osv.
	int maxInhomogenity;

	///vre grense for antall iterasjoner som glatter bildet
	int maxBlurIterations;

	///Nedre grense for areal med tanke p akseptable regionsegmenteringer
	int minArea;

	///Verdi som angir om region growing skal igangsettes for  spise opp usegmenterte piksler eller ikke
	int grow;

	///Verdi som angir om segmenteringsresultatene skal skrives ut til fil eller ikke
	int writeToFile;

public:

	/**
	 * @brief Konstruktor
	 *
	 * @param _minIntensity En terskel som angir et minstekrav til fargeintensitet. Piksler med 
	 *				   	    intensiteter (r*r+g*g+b*b) mindre enn denne terskelen vil ikke bidra
	 *						i segmenteringsprosessen.
	 * @param _maxInhomogenity En terskel som angir maksimalt tillatte inhomogenitet i regioner. 
	 *						   Dvs at pikslene P1 og P2 bare sles sammen til en region dersom 
	 *						   |r1-r2| + |g1-g2| + |b1-b2| <= _maxInhomogenity, der r1 og r2 er 
	 *						   den rde intensiteten til henholdsvis piksel P1 og P2 osv.
	 * @param _maxBlurIterations vre grense for antall iterasjoner som glatter bildet.
	 * @param _minArea Nedre grense for areal med tanke p akseptable regionsegmenteringer.
	 * @param _grow Verdi som angir om region growing skal igangsettes for  spise opp usegmenterte
	 *			    piksler eller ikke.
	 * @param _writeToFile Verdi som angir om segmenteringsresultatene skal skrives ut til fil 
						   eller ikke.
	 */
	ColorSegment(int _minIntensity, int _maxInhomogenity, int _maxBlurIterations, 
				 int _minArea, int _grow, int _writeToFile);


	/**
	 * @brief Destruktor
	 */
	~ColorSegment();


	/**
	* @brief Metode som segmenterer og returnerer et image-objekt av et bilde.
	* 
	* @param filename Navn p bilde som skal segmenteres
	* @param index Lengde p stien (path) hvor det aktuelle bildet ligger lagret
	*
	* @return Image objekt av bildet.
	*/
	Image * segment(const char * filename, int index);


	/**
	* @brief Metode som henter dimensjonen til et bilde
	*
	* @param filename Navn p inputbilde
	*
	* @return Peker til en vektor av lengde 2, der frste og andre rad henholdsvis inneholder
	*		  bildets bredde og hyde.
	*/
	int * getDimension(const char * filename);

};