
//Refereanser til eksternt grensesnitt/kode som anvendes i denne filen
#include "Feature.h"
#include <iostream>


/** 
 * @brief Finner absoluttverdien til aktuelle flyttal.
 *
 * @param value Flyttallet man nsker absoluttverdien av.
 *
 * @return Flyttallets absouluttverdi.
 */
float Feature::absoluteValue(float value)
{
	if ( value < 0 ) return -1 * value;
	return value;
}



/**
 * @brief Garanterer verdier mellom en nedre og vre definert grense.
 * 
 * @param Flyttallet som skal hndteres.
 *
 * @return Verdi innenfor definert omrde.
 */
float Feature::valueHandler(float value)
{
	if ( value < -INFINITY )
	{
		value = -INFINITY;
	} else if ( value > INFINITY )
	{
		value = INFINITY;
	}
	return value;
}



/** 
 * @brief Konstruktor
 */
Feature::Feature()
{
	//Tom konstruktor
}



/** 
 * @brief Konstruktor
 */
Feature::~Feature()
{
	//Tom destruktor
}



/** 
 * @brief Initialiserer egenskapskandidaten.
 *
 * @param abc Dobbel pekerstruktur (vektor v av 2-dimensjonale vektorer w), der 
 *	          vektor v sin dimensjon tilsvarer antall dimensjoner i egenskapskandidaten,
 *		      og vektorene w hver har hovedkrav 1 og 2 som komponentverdier.
 */
void Feature::initialize(float **abc)
{
	homogenMax = new float[DIM]; 
	homogenMean = new float[DIM];
	inhomogenMean = new float[DIM];
	req1 = new float[DIM];
	req2 = new float[DIM];
	for (int i=0;i<DIM;i++)
	{
		homogenMax[i] = -INFINITY;
		homogenMean[i] = 0;
		inhomogenMean[i] = 0;
		req1[i] = abc[i][0];
		req2[i] = abc[i][1];
	}
}



/** 
 * @brief Henter egenskapskandidatens identifikasjonen.
 *
 * @return Egenskapskandidatens identifikasjonen.
 */
int Feature::getId()
{
	return ID;
}



/** 
 * @brief Henter egenskapskandidatens navn.
 *
 * @return Egenskapskandidatens navn.
 */
const char * Feature::getName()
{
	return NAME;
}



/** 
 * @brief Henter egenskapskandidatens krav til dimensjon.
 *
 * @return Egenskapskandidatens dimensjon.
 */
int Feature::getDim()
{
	return DIM;
}



/** 
 * @brief Henter egenskapskandidatens poengsum
 *
 * @return Egenskapskandidatens poengsum
 */
int Feature::getCredit()
{
	for (int i=0; i<DIM; i++)
	{
		float prop1 = homogenMean[i] / homogenMax[i];
		float prop2 = absoluteValue( inhomogenMean[i] - homogenMean[i] ) / inhomogenMean[i];
		if ( prop1 < req1[i] || prop2 < req2[i] ) return 0;
	}
	return 1;
}



/** 
 * @brief Beregner differansen av egenskapskandidatens kalkulerte resultater av 
 *        to aktuelle regioner.
 *
 * @param r1 Peker til den ene regionen.
 * @param r2 Peker til den andre regionen.
 *
 * @return Peker til en vektor av flytverdier av ulikhet mellom de to regionene.
 */
float * Feature::difference(Region *r1, Region *r2)
{
	float *diff = new float[DIM];
	for (int i=0; i<DIM; i++)
	{
		diff[i] = absoluteValue( r1->getFeatureValue(ID,i) - r2->getFeatureValue(ID,i) );
	}
	return diff;
}



/** 
 * @brief Adderer egenskapskandidatens differanseverdier av to regioner til variablene 
 *		  homogenMean eller inhomogenMean, for  senere kunne beregne
 *		  gjennomsnittlige beregninger.
 *
 * @param homogen Boolsk verdi som ved (true) forteller om det er to godkjente 
 *                bilders objekter som vurderes, og som ved (false) forteller
 *				  at det er objekter av et gokjent og et underkjent bilde som
 *				  vurders.
 * @param diff Peker til differansen mellom denne egenskapskandidatens beregninger
 *			   for to regioner.
 */
void Feature::add(bool homogen, float *diff)
{
	if ( homogen )
	{
		for (int i=0;i<DIM;i++)
		{
			homogenMean[i] += diff[i];
		}
	}
	else
	{
		for (int i=0;i<DIM;i++)
		{
			inhomogenMean[i] += diff[i];
		}
	}
}



/** 
 * @brief Oppdaterer pekeren som holder oversikt over maksimal forskjell mellom 
 *		  objekter av godkjente bilders regioner seg imellom.
 *
 * @param diff Peker til differansen mellom denne egenskapskandidatens beregninger
 *			   for to regioner.
 */
void Feature::maxHomogen(float *diff)
{
	for (int i=0;i<DIM;i++)
	{
		if ( diff[i] > homogenMax[i] )
		{
			homogenMax[i] = diff[i];
		}
	}
}



/** 
 * @brief Beregner gjennomsnittlig regiondifferanseverdi for denne egenskapskandidaten.
 *
 * @param homogen Boolsk verdi som ved (true) forteller om det er to godkjente 
 *                bilders objekter som vurderes, og som ved (false) forteller
 *				  at det er objekter av et gokjent og et underkjent bilde som
 *				  vurders.
 * @param divisor Divisor for  rekne gjennomsnittlige differanseverdier for denne
 *				  egenskapskandidaten.
 */
void Feature::divide(bool homogen, int divisor)
{

	if ( homogen )
	{
		for (int i=0;i<DIM;i++)
		{
			homogenMean[i] /= divisor;
		}
	}
	else
	{
		for (int i=0;i<DIM;i++)
		{
			inhomogenMean[i] /= divisor;					
		}
	}
}