
//Refereanser til eksternt grensesnitt/kode som anvendes i denne filen
#include <cmath>
#include "Feature_FD.h"

//Denne kilden bruker kode definert i navnerommet std
using namespace std;

//Definerer konstanter
const int P = 1;
const float PI = -3.14159;


/**
 * @brief Konstruktor
 */
Feature_FD::Feature_FD()
{
	Feature();
	ID = 1;
	NAME = "Fourier Descriptors";
	DIM = 2 * P;
}



/**
 * @brief Destruktor
 */
Feature_FD::~Feature_FD() 
{
	//Tom destruktor
}



/**
 * @brief Beregner Fourier Descriptor til en regions kontur.
 * 
 * @param r Peker til aktuell region.
 *
 * @return Peker til en vektor som holder beregnet Fourier Descriptor av regionens kontur.
 */
float * Feature_FD::calculate(Region *r)
{
	float f =  1.0 / (float) r->getBorder().size();
	float *a = new float[DIM];
	for (int u=0; u<P; u++)
	{
		int indexRe = 2 * u;
		int indexIm = indexRe + 1;
		for (int k=0; k<r->getBorder().size(); k++)
		{
			float tetta = -2 * PI * k * u * f;
			float cosTetta = cos(tetta);
			float sinTetta = sin(tetta);
			a[indexRe] += r->getBorder().at(k).x * cosTetta - r->getBorder().at(k).y * sinTetta;
			a[indexIm] += r->getBorder().at(k).x * sinTetta + r->getBorder().at(k).y * cosTetta;
		}
		a[indexRe] = valueHandler( f * a[indexRe] );
		a[indexIm] = valueHandler( f * a[indexIm] );		
	}
	return a;
}