
//Refereanser til eksternt grensesnitt/kode som anvendes i denne filen
#include <cmath>
#include <vector>
#include "Feature_Moments.h"

//Denne kilden bruker kode definert i navnerommet std
using namespace std;


/**
 * @brief Beregner sentralmomentet.
 * 
 * @param p Indeks p i Mpq.
 * @param q Indeks q i Mpq.
 * @param r Peker til region-objekt.
 *
 * @return Sentralmomentet.
 */
int Feature_Moments::centralMoment(int p, int q, Region *r)
{
	int sum = 0;
	vector<Pixel> pixels = r->getPixels();
	for (int i=0;i<r->getArea();i++)
	{
		Pixel pix = pixels.at(i);
		int x = pix.x - r->getCentroid().x;
		int y = pix.y - r->getCentroid().y;
		sum += (int) ( pow(x,p) + pow(y,q) );		
	}
	return sum;
}



/**
 * @brief Beregner skaleringsinvarianse momenter.
 * 
 * @param p Indeks p i Mpq.
 * @param q Indeks q i Mpq.
 * @param r Peker til region-objekt.
 *
 * @return Skaleringsinvariant momentverdi
 */
float Feature_Moments::scaleInvariance(int p, int q, Region *r)
{
	return (float) ( centralMoment(p,q,r) / pow( r->getArea(),((p+q)/2+1) ) );
}



/**
 * @brief Konstruktor
 */
Feature_Moments::Feature_Moments()
{
	Feature();
	ID = 0;
	NAME = "Statistical moments";
	DIM = 7;
}



/**
 * @brief Destruktor
 */
Feature_Moments::~Feature_Moments()
{
	//Tom destruktor
}



/**
 * @brief Beregner statistiske momenter til en region.
 * 
 * @param r Peker til aktuell region.
 *
 * @return Peker til en vektor som holder beregnede statistiske momenter av regionen.
 */
float * Feature_Moments::calculate(Region *r)
{
	float eta02 = scaleInvariance(0,2,r);
	float eta03 = scaleInvariance(0,3,r);
	float eta11 = scaleInvariance(1,1,r);
	float eta12 = scaleInvariance(1,2,r);
	float eta20 = scaleInvariance(2,0,r);
	float eta21 = scaleInvariance(2,1,r);
	float eta30 = scaleInvariance(3,0,r);

	float * moments = new float[7];

	moments[0] = valueHandler( eta20+eta02 );
	moments[1] = valueHandler( pow(eta20-eta02,2) + 4*pow(eta11,2) );
	moments[2] = valueHandler( pow(eta30-3*eta12,2) + pow(3*eta21-eta03,2) );
	moments[3] = valueHandler( pow(eta30+eta12,2) + pow(eta21+eta03,2) );
	moments[4] = valueHandler( (eta30-3*eta12) * (eta30+eta12) * 
							   ( pow(eta30+eta12,2) - 3*pow(eta21+eta03,2) ) +
							   (3*eta21-eta03) * (eta21+eta03) *
							   ( 3*pow(eta30+eta12,2) - pow(eta21+eta03,2) ) );
    moments[5] = valueHandler( (eta20-eta02) * 
							   ( pow(eta30+eta12,2) - pow(eta21+eta03,2) ) +
							   4 * eta11 * (eta30+eta12) * (eta21 + eta03) );
	moments[6] =  valueHandler( (3*eta21-eta03) * (eta30+eta12) * 
							    ( pow(eta30+eta12,2) - 3*pow(eta21+eta03,2) ) +
							    (3*eta12-eta30) * (eta21+eta03) *
								( 3*pow(eta30+eta12,2) - pow(eta21+eta03,2) ) );
	return moments;
}