#pragma once

//Refereanser til eksternt grensesnitt/kode som anvendes i denne filen
#include "Feature.h"
#include "Region.h"


/**
 * @class Feature_Moments
 * @brief Beregner statistiske momenter til en region.
 *
 * Klassen arver funksjonalitet fra Feature-klassen.
 *
 * @version 1.0
 * @author Per Gunnar Brdsen
 */
class Feature_Moments: public Feature
{

	/**
	 * @brief Beregner sentralmomentet.
	 * 
	 * @param p Indeks p i Mpq.
	 * @param q Indeks q i Mpq.
	 * @param r Peker til region-objekt.
	 *
	 * @return Sentralmomentet.
	 */
	int centralMoment(int p, int q, Region *r);


	/**
	 * @brief Beregner skaleringsinvarianse momenter.
	 * 
	 * @param p Indeks p i Mpq.
	 * @param q Indeks q i Mpq.
	 * @param r Peker til region-objekt.
	 *
	 * @return Skaleringsinvariant momentverdi
	 */
	float scaleInvariance(int p, int q, Region *r);

public:

	/**
	 * @brief Konstruktor
	 */
	Feature_Moments();

	/**
	 * @brief Destruktor
	 */
	~Feature_Moments();

	/**
	 * @brief Beregner statistiske momenter til en region.
	 * 
	 * @param r Peker til aktuell region.
	 *
	 * @return Peker til en vektor som holder beregnede statistiske momenter av regionen.
	 */
	float * calculate(Region *r);
};
