
//Refereanser til eksternt grensesnitt/kode som anvendes i denne filen
#include <cmath>
#include "Feature_Radius.h"

//Denne kilden bruker kode definert i navnerommet std
using namespace std;


/**
 * @brief Konstruktor
 */
Feature_Radius::Feature_Radius()
{
	Feature();
	ID = 7;
	NAME = "Radius";
	DIM = 3;
}



/**
 * @brief Destruktor
 */
Feature_Radius::~Feature_Radius() 
{
	//Tom destruktor
}



/**
 * @brief Beregner minste, strste og gjennomsnittlige "radius" til en region.
 * 
 * @param r Peker til aktuell region.
 *
 * @return Peker til en vektor som holder beregnet "radius" av regionen.
 */
float * Feature_Radius::calculate(Region *r)
{
	float min = INFINITY;
	float max = 0;
	float sum = 0;

	for (unsigned int i=0; i<r->getBorder().size(); i++)
	{	
		int deltaX = r->getBorder().at(i).x - r->getCentroid().x;
		int deltaY = r->getBorder().at(i).y - r->getCentroid().y;
		float rSquare = (float) ( deltaX * deltaX + deltaY * deltaY );
		if ( rSquare < min ) 
		{ 
			min = rSquare;
		}
		if ( rSquare > max ) 
		{ 
			max = rSquare;
		}
		sum += sqrt(rSquare);
	}

	float *v = new float[DIM];
	
	if ( r->getBorder().size() > 0 )
	{
		v[0] = (float) sqrt( valueHandler(min) );
		v[1] = (float) sqrt( valueHandler(max) );
		v[2] = sum / r->getBorder().size();
	}
	else
	{	
		v[0] = 0;
		v[1] = 0;
		v[2] = 0;
	}

	return v;

}