
//Refereanser til eksternt grensesnitt/kode som anvendes i denne filen
#include "Feature_Symmetry.h"

//Definerer konstanter
const bool MARKED = true;
const bool UNMARKED = false;


/**
 * @brief Konstruktor
 */
Feature_Symmetry::Feature_Symmetry()
{
	Feature();
	ID = 8;
	NAME = "Symmetry";
	DIM = 1;
}



/**
 * @brief Destruktor
 */
Feature_Symmetry::~Feature_Symmetry() 
{
	//Tom destruktor
}



/**
 * @brief Beregner symmetrien til en regions kontur.
 * 
 * @param r Peker til aktuell region.
 *
 * @return Peker til en vektor som holder beregnet symmetri av regionens kontur.
 */
float * Feature_Symmetry::calculate(Region *r)
{
	int *bb = r->getBoundingBox();
	int halfW = max( r->getCentroid().x - bb[3], bb[1] - r->getCentroid().x );
	int halfH = max( r->getCentroid().y - bb[0], bb[2] - r->getCentroid().y );
	int w = 2 * halfW + 1;
	int h = 2 * halfH + 1;
	int s = w * h;
	bool *a = new bool[s];
	float *v = new float[1];
	v[0] = 0;

	for (int i=0; i<s; i++)
	{	
		a[i] = UNMARKED;
	}

	for (int i=0; i<r->getBorder().size(); i++)
	{	
		int j = ( r->getBorder().at(i).y - bb[0] ) * w + r->getBorder().at(i).x - bb[3];
		a[j] = MARKED;
	}

    for (int i=0; i<h; i++)
	{
		for (int j=0; j<halfW; j++)
		{
			if ( a[ i*w + j ] != a[ i*w + w - j] ) v[0]++;
		}
	}

    for (int i=0; i<w; i++)
	{
		for (int j=0; j<halfH; j++)
		{
			if ( a[ j*w + i] != a[ (h-j-1)*w + i ] ) v[0]++;
		}
	}

	delete[] a;
	v[0] = valueHandler( v[0] / (float) r->getBorder().size() );

	return v;
}