#pragma once

//Refereanser til ekstern kode som anvendes i denne filen
#include <vector>
#include "Region.h"

//Denne kilden bruker kode definert i navnerommet std
using namespace std;

/**
 * @class Image
 * @brief Klasse som holder bildeinformasjon.
 * 
 * @version 1.0
 * @author Per Gunnar Brdsen
 */
class Image
{

	///Bildefilens navn
	const char *name;

	///Peker til dynamisk vektor bestende av pekere til region objekter i bildet.
	vector<Region*> *regions;

public:

	/** 
	 * @brief Konstruktor
	 * 
	 * @param _regions Peker til dynamisk vektor bestende av pekere til region
	 *				   objekter i bildet.
	 * @param _name Peker til bildefilens navn.
	 */
	Image(vector<Region*> *_regions, const char *_name);


	/** 
	 * @brief Destruktor
	 */
	~Image();


	/**
	 * @brief Henter navnet til filen som inneholder det aktuelle bildet.
	 *
	 * @return Navn til filen som inneholder det aktuelle bildet.
	 */
	const char * getName();


	/**
	 * @brief Henter en dynamisk vektor over pekere til regionobjekter i bildet.
	 *
	 * @return Dynamisk vektor bestende av pekere til regionobjekter i bildet.
	 */
	vector<Region*> *getRegions();

};
