
//Refereanser til eksternt grensesnitt/kode som anvendes i denne filen
#include <cmath>
#include "Image_pair.h"

//Denne kilden bruker kode definert i navnerommet std
using namespace std;


/**
 * @brief Kvadrerer en heltallsverdi.
 *
 * @param x Heltallsverdi som skal kvadreres.
 *
 * @return Kvadratet av heltallsverdien.
 */
int kvadrat(int x)
{
	return x*x;
}



/**
 * @brief Kvadratroten av en heltallsverdi.
 *
 * @param x Heltallsverdi som kvadratroten nskes funnet av.
 *
 * @return Kvadratroten av heltallsverdien.
 */
float rot(int x)
{
	return std::sqrt( (float) x );
}



/**
 * @brief Konstruktor
 */
Image_pair::Image_pair()
{
	//Tom konstruktor
}



/**
 * @brief Konstruktor
 *
 * @param _image1 Peker til den ene bildet i bildeparet.
 * @param _image2 Peker til det andre bildet i bildeparet.
 * @param _homogenious Boolsk verdi som forteller om bildeparet bestr av to godkjente 
 *                     bilder (true), eller ett godkjent og ett underkjent bilde (false).
 * @param weight Peker til en vektor som inneholder vektingen av den lille 
 *               egenskapsmengden (areal, massesenter og omlsuttende rektangel).
 * @param sort Peker til sorteringsobjektet sort.
 */
Image_pair::Image_pair(Image *_image1, Image *_image2, bool _homogenious, float *weight, Sort *sort)
{
	image1 = _image1;
	image2 = _image2;
	homogenious = _homogenious;
	calculateBestCorrespondenceVector( calculateComparisonMatrix(weight), sort );
}



/**
 * @brief Destruktor
 */
Image_pair::~Image_pair()
{
	//Tom destruktor
}



/**
 * @brief Beregner sammenlikningsgrunnlaget for mapping av regioner i bildeparet.
 *
 * @param weight Peker til en vektor som inneholder vektingen av den lille 
 *				 egenskapsmengden (areal, massesenter og omlsuttende rektangel).
 *
 * @return Peker til vektor som inneholder sammenlikningsgrunnlaget for
 *		   mapping av regioner i bildeparet.
 */
int * Image_pair::calculateComparisonMatrix(float * weight)
{
	int *comparison = new int[ image1->getRegions()->size() * image2->getRegions()->size() ];
	for (int unsigned i=0;i<image1->getRegions()->size();i++)
	{
		Region *r1 = image1->getRegions()->at(i);
		for (int unsigned j=0;j<image2->getRegions()->size();j++)
		{
			Region *r2 = image2->getRegions()->at(j);
			int diffArea = abs( r1->getArea() - r2->getArea() );
			float diffCentroid = rot( kvadrat( (signed int) (r1->getCentroid().x - r2->getCentroid().x) ) +
									   kvadrat( (signed int) (r1->getCentroid().y - r2->getCentroid().y) ) );
			int * bb1 = r1->getBoundingBox();
			int * bb2 = r2->getBoundingBox();
			int diffBoundingBox = abs( bb1[0] - bb2[0] ) + abs( bb1[1] - bb2[1] ) + 
								  abs( bb1[2] - bb2[2] ) + abs( bb1[3] - bb2[3] );

			comparison[ i*image2->getRegions()->size() + j ] =  (int) ( weight[0] * diffArea +
																	    weight[1] * diffCentroid +
																	    weight[2] * diffBoundingBox );
		}
	}
	return comparison;
}



/**
 * @brief Beregner beste korrespondans/mapping mellom regioner i bildeparet.
 *
 * @param comparison Peker til vektor som inneholder sammenlikningsgrunnlaget
 *					 for mapping av regioner i bildeparet.
 * @param sort Peker til sorteringsobjektet sort.
 */
void Image_pair::calculateBestCorrespondenceVector(int *comparison, Sort *sort)
{	
	int w = image2->getRegions()->size();
	int h = image1->getRegions()->size();
	bestCorrespondenceVector = sort->calculateBestCorrespondence(comparison, w, h);
	delete[] comparison;
}



/**
 * @brief Henter den beste korrespondans/mapping mellom regioner i bildeparet.
 *
 * @return Peker til vektor som inneholder den beste mappingen/korrespondansen
 *		   mellom regioner i bildeparet.
 */
int * Image_pair::getBestCorrespondenceVector()
{
	return bestCorrespondenceVector;
}



/**
 * @brief Henter det bildet av bildeparet som har minst antall regioner.
 *
 * @return Peker til det bildet av bildeparet som har minst antall regioner.
 */
Image * Image_pair::getSmallImage()
{
	return image1;
}



/**
 * @brief Henter det bildet av bildeparet som har strst antall regioner.
 *
 * @return Peker til det bildet av bildeparet som har strst antall regioner.
 */
Image * Image_pair::getBigImage()
{
	return image2;
}


/**
 * @brief Henter ut informasjon om bildeparet er homogent eller ikke.
 *
 * @return True dersom bildeparet bestr av to godkjente bilder, false ellers. 
 */
bool Image_pair::getHomogenious()
{
	return homogenious;
}