
//Refereanser til eksternt grensesnitt/kode som anvendes i denne filen
#include "NN_connection.h"


/** 
 * @brief Konstruktor
 */
NN_connection::NN_connection()
{
	//Tom konstruktor
}



/** 
 * @brief Destruktor
 */
NN_connection::~NN_connection()
{
	//Tom destruktor
}



/** 
 * @brief Oppretter et standard og fullt koblet backpropagation nevralt nettverk.
 *
 * @param numInput Antall noder i inputlaget.
 * @param numNeuronsHidden Antall noder i de skjulte lagene.
 * @param numOutput Antall noder i outputlaget.
 */
void NN_connection::createNetwork(int numInput, int numNeuronsHidden, int numOutput)
{
	ann = fann_create_standard(3, numInput, numNeuronsHidden, numOutput);
	fann_randomize_weights(ann, -0.1, 0.1);
}



/** 
 * @brief Oppretter et nevralt nettverk fra fil.
 *
 * @param inputFile Peker til navn p fil som inneholder det nevrale nettverket.
 */
void NN_connection::createNetworkFromFile(const char *inputFile)
{
	ann = fann_create_from_file(inputFile);
}



/** 
 * @brief Trener epokevis det nevrale nettverket med en datamengde som leses fra fil.
 *
 * @param input Peker til navn p fil som inneholder treningsdataene.
 * @param maxEpochs Maksimalt antall treningsepoker.
 * @param epochsBetweenReports Antall epoker mellom hver utskrift av statusrapport til skjerm.
 * @param desiredError vre akseptable feilgrense ved trening for det nevrale nettverket 
 *					   (treningsfasen er over nr nettets estimererte feil er mindre enn denne).
 * @param output Peker til navn p fil som inneholder det ferdigtrenede nettverket.
 */
void NN_connection::train(const char *input, int maxEpochs, int epochsBetweenReports, 
						  float desiredError, const char *output)
{
	fann_set_activation_function_hidden(ann, FANN_SIGMOID_SYMMETRIC);
	fann_set_activation_function_output(ann, FANN_SIGMOID_SYMMETRIC);
	fann_train_on_file(ann, input, maxEpochs, epochsBetweenReports, desiredError);
	fann_save(ann, output);
	destroy();
}



/** 
 * @brief Eksekverer det nevrale nettverket med en inputvektor.
 *
 * @param vector Peker til vektor som inneholder inputvektor med testdata.
 *
 * @return 1 hvis inputvektor godkjennes, og 0 hvis inputvektor underkjennes.
 */
int NN_connection::execute(float *vector)
{
	float *res = fann_run(ann, vector);  
	if ( res[0] > 0 ) return 1;
	return 0;
}



/** 
 * @brief Sletter det nevrale nettverket fra minnet.
 */
void NN_connection::destroy()
{
	fann_destroy(ann);
}