#pragma once

//Refereanser til eksternt grensesnitt/kode som anvendes i denne filen
#include "Fann.h"


/**
 * @class NN_connection
 * @brief Klasse som tar for seg koblinger mot det nevrale nettverket.
 *
 * Denne koden anvender Fast Artificial Neural Network Library (FANN) version 2.0.0. 
 * Mer informasjon om dette glimrende biblioteket finnes p Fanns websider:
 * http://leenissen.dk/fann.
 *
 * @version 1.0
 * @author Per Gunnar Brdsen
 */
class NN_connection
{

	///Peker til objekt som representerer det nevrale nettverket. 
	struct fann *ann;

public:

	/** 
	 * @brief Konstruktor
	 */
	NN_connection();


	/** 
	 * @brief Destruktor
	 */
	~NN_connection();


	/** 
	 * @brief Oppretter et standard og fullt koblet backpropagation nevralt nettverk.
	 *
	 * @param numInput Antall noder i inputlaget.
	 * @param numNeuronsHidden Antall noder i de skjulte lagene.
	 * @param numOutput Antall noder i outputlaget.
	 */
	void createNetwork(int numInput, int numNeuronsHidden, int numOutput);


	/** 
	 * @brief Oppretter et nevralt nettverk fra fil.
	 *
	 * @param inputFile Peker til navn p fil som inneholder det nevrale nettverket.
	 */
	void createNetworkFromFile(const char *inputFile);


	/** 
	 * @brief Trener epokevis det nevrale nettverket med en datamengde som leses fra fil.
	 *
	 * @param input Peker til navn p fil som inneholder treningsdataene.
	 * @param maxEpochs Maksimalt antall treningsepoker.
	 * @param epochsBetweenReports Antall epoker mellom hver utskrift av statusrapport til skjerm.
	 * @param desiredError vre akseptable feilgrense ved trening for det nevrale nettverket 
	 *					   (treningsfasen er over nr nettets estimererte feil er mindre enn denne).
	 * @param output Peker til navn p fil som inneholder det ferdigtrenede nettverket.
	 */
	void train(const char *input, int maxEpochs, int epochsBetweenReports,
			   float desiredError, const char *output);


	/** 
	 * @brief Eksekverer det nevrale nettverket med en inputvektor.
	 *
	 * @param vector Peker til vektor som inneholder inputvektor med testdata.
	 *
	 * @return 1 hvis inputvektor godkjennes, og 0 hvis inputvektor underkjennes.
	 */
	int execute(float *vector);


	/** 
	 * @brief Sletter det nevrale nettverket fra minnet.
	 */
	void destroy();
};