
//Refereanser til eksternt grensesnitt/kode som anvendes i denne filen
#include <vector>
#include "Sort.h"

//Denne kilden bruker kode definert i navnerommet std
using namespace std;



/**
 * @class Quicksort
 * @brief En "in-place"-implementasjon av quicksort.
 *
 * Denne templaten er en "in-place"-implementasjon av quicksort. Mer informasjon 
 * om quicksort algoritmen gis blant annet p Wikipedia: 
 * http://en.wikipedia.org/wiki/Quicksort
 *
 * Template-klassens kildekode er hentet fra:
 * http://users.sdsc.edu/~decastro/home/projects/datastruct/quicksort.cpp
 *
 * Undertegnede har utvidet klassen til  behandle en ekstra parameter (b),
 * som representerer regionenes opprinnelige kolonneposisjon i inputvektoren (a).
 * Dette gjres for  holde styr p regionenes "identifikasjon", siden 
 * "in-place"-sorteringen flytter p verdiene i inputvektoren a. Verdier flyttes 
 * helt identisk innad i vektor a og b.
 *
 * @param a Peker til vektor som inneholder de verdiene som skal sorteres.
 * @param b Peker til vektor som inneholder kolonneposisjonene til 
 *			regionene som sorteres.
 * @param l Nedre indeksgrense for hvor sortering skal finne sted. 
 * @param r vre indeksgrense for hvor sortering skal finne sted.
 * @param s Lengde p inputvektorene a og b.
 *
 * @version 1.0
 * @author Alex DeCastro, http://users.sdsc.edu/~decastro/home/projects/datastruct/quicksort.cpp
 * @author Modifisert og tilpasset av Per Gunnar Brdsen
 *
 */
template<class T>
void Quicksort(T *a, T *b, int l, int r, int s)
{
	//Returner hvis venstre pekerindeks har ndd igjen eller overstiger hyre pekerindeks
    if (l >= r) return;

	//Definerer variabler
    int i, j;
    int num_left, num_right;
    T pivot, temp;

	//Iterer s lenge som venstre pekerindeks er mindre enn hyre pekeringeks
    while (l < r)
    {

		//Setter pivotelement og startverdier for indekser
        i = l;
        j = r + 1;
        pivot = a[l];

        while (true)
        {
			//Inkrementer indeks i helt til pivot-elementet er mindre eller lik
			//veltorverdien av indeks i (eller enden av vektoren er ndd)
            do
            {
              i = i + 1;
			}
			while ( a[i]<pivot && i<s );
            
			//Dekrementer indeks j helt til pivot-elementet er strre eller lik
			//vektorverdien av indeks j
            do
            {
                j = j - 1;
            }
            while (a[j] > pivot);
            
			//Avbryt lkke dersom indeks i>=j
            if (i >= j) break;

			//Bytter plass mellom mappingverdiene i vektor a gitt av indeks i og j
            temp = a[i];
            a[i] = a[j];
            a[j] = temp;

			//Bytter plass mellom kolonneverdiene i vektor b gitt av indeks i og j
			temp = b[i];
			b[i] = b[j];
			b[j] = temp;
        };
        
		//Setter verdien i venstre indeks til laveste verdi
        a[l] = a[j];

		//Setter verdien i indeks j til pivotelementets verdi, da alle lavere indekser inneholder lavere
		//verdier enn pivotelementet)
        a[j] = pivot;

		//Flytter kolonneverdiene i vektor b p helt tilsvarende mte
		temp = b[l];
		b[l] = b[j];
		b[j] = temp;

		//Variabler som kreves for  avgjre om venstre eller hyre del skal benyttes i rekursiv beregning
        num_left  = (j-1) - l;
        num_right = r - (j+1);
        
        if (num_left <= num_right)
        {
			//Rekursivt kall for indeks j-1 og nedover
            Quicksort(a, b, l, j-1, s);

			//Oppdaterer venstre pekerindeks, siden det rekursive kallet ovenfor har sortert opp til indeks j
            l = j+1;
        }
        else
        {
			//Rekursivt kall for indeks j+1 og oppover
            Quicksort(a, b, j+1, r, s);

			//Oppdaterer hyre pekerindeks, siden det rekursive kallet ovenfor har sortert ned til indeks j
            r = j-1;
        }
    }
}



/**
 * @brief Metode som etter grdighetsprinsippet returnerer beste mapping fra en
 *		  region i et bilde til en av flere mulige regioner i bildeparets andre bilde.
 *
 * Parameterforklaring:
 * La (M,S) vre et bildepar, der M og S henholdsvis betegner bildet med minst og strst
 * antall regioner.
 *
 * @param sortedRows Peker til vektor som inneholder ferdigsorterte/prioriterte
 *		             mapperelasjoner mellom regioner i M og S.
 * @param col Peker til vektor med boolske verdier, som angir om aktuelle regioner 
			  i S er mappet fra fr av eller ikke.
 * @param row Betegner aktuell region i M.
 * @param width Antall regioner i S.
 *
 * @return Returnerer etter grdighetsprinsippet den beste mappingen fra en region i M
 *		   til en av flere mulige regioner i S.
 */
int getFrontValue(int *sortedRows, bool *col, int row, int width)
{	
	//Iterer gjennom alle regioner i S
	for (int i=0;i<width;i++)
	{
		//La c betegne aktuell kolonne (region i S)
		int c = sortedRows[ row*width + i ];

		//Returner den frste kolonnen (regionen i S) som er ikke er mappet fra fr av
		if ( col[c] ) return c;
	}
	return -1;
}



/**
 * @brief Metode som beregner annenrangs mappinger.
 *
 * Metode som beregner annenrangs mappinger i tilfeller nr bildene i bildeparet ikke har 
 * samme antall regioner. La (M,S) vre et bildepar, der M og S henholdsvis betegner bildet 
 * med minst (n stk) og strst (m stk) antall regioner. I tilfeller der m>n brukes denne 
 * metoden til  mappe m-n stk regioner, slik at alle regioner i S blir mappet en gang. En 
 * region i M kan da mappes til mer enn en region i S. Denne metoden brukes av 
 * calculateBestCorrespondence(...)-metoden.
 *
 * For mer informasjon se prosjektrapport kapittel 6.1.2 og 6.1.3:
 * http://www.idi.ntnu.no/~pergunb/rapport.pdf
 *
 * @param input Peker til vektor som representerer sammenlikningsgrunnlaget. Vektorens 
 *			    strrelse er da n*m. Alle regioner i M er sammenliknet mot alle regioner i
 *				S, og hver slik sammenlikning har sin posisjon i vektoren.
 * @param col Peker til vektor med boolske verdier som angir om kolonnen med aktuell indeks
 *		      er mappet eller ikke.
 * @param width Antall regioner i M.
 * @param height Antall regioner i S.
 * @param output Peker til vektor som holder den beste korresponansen/mappin mellom regioner i
 *				 i bildeparet. Denne vektoren vil ha verdien 1 i de posisjoner der to regioner
 *				 mappes, og verdien 0 for alle de andre posisjonene. La for eksempel M best 
 *				 av 3 regioner og S best av 4 regioner. Dersom vektorverdien i indeks 5 da har 
 *				 verdien 1, betyr dette at region nr 2 i M er mappet med region nr 2 i S.
 *
 * @return Peker til output-vektoren etter at metoden har oppdatert denne med annenrangs 
 *		   mappinger.
 */
int * secondOrderMapping(int *input, bool *col, int width, int height, int *output)
{
	//Iterer gjennom alle regioner i S
	for (int i=0;i<width;i++)
	{
		//Hopp til neste iterasjon dersom regionen er mappet fra fr av
		if ( !col[i] ) continue;

		//Oppretter variabler og startverdier
		int min = INFINITY_INT;
		int index_r = 0;

		//Iterer gjennom alle regioner i M
		for (int j=0;j<height;j++)
		{
			//Dersom aktuell mapping gitt av raden j og kolonnen i er den beste s langt
			if ( input[ j*width + i ] < min )
			{
				//Sett variabelen min til denne mappingens verdi
				min = input[ j*width + i ];

				//La index_r betegne aktuell rad (region i M)
				index_r = j;
			}
		}

		//Merk av mapping i outputvektor p den beste posisjonen som ble funnet
		output[ index_r * width + i ] = 1;
	}

	//Returnerer optadert outputvektor
	return output;
}



/**
 * @brief Konstruktor
 */
Sort::Sort() 
{
	//Tom konstruktor
}



/**
 * @brief Destruktor
 */
Sort::~Sort() 
{
	//Tom destruktor
}



/**
 * @brief Beregner beste korrespondans/mapping mellom regioner i et bildepar.
 *
 * Metode som beregner den beste korrespondanse/mapping mellom regioner i et bildepar. 
 * Beregningen gjres p bakgrunn av et sammenlikningsgrunnlag, der differansen av ulike 
 * regioners beregninger av den lille egenskapsmengden (areal, massesenter og omsluttende 
 * rektangel) mles.
 *
 * @param input Vektor som representerer sammenlikningsgrunnlaget. La (M,S) vre et bildepar,
 * der M og S henholdsvis betegner bildet med minst (n stk) og strst (m stk) antall regioner.
 * Vektorens strrelse er da lik antall regioner i M multiplisert med antall regioner i S (n*m). 
 * Alle regioner i M er sammenliknet mot alle regioner i S, og hver slik sammenlikning har sin 
 * posisjon i vektoren.
 *
 * @param input Peker til vektor som inneholder sammenlikningsgrunnlaget for mapping av regioner i bildeparet.
 * @param width Antall regioner i S (m stk).
 * @param height Antall regioner i M (s stk).
 *
 * @return En peker til en vektor som holder den beste korrespondansen mellom regioner i bildeparet.
 *		   Denne vektoren vil ha verdien 1 i de posisjoner der to regioner mappes, og verdien 0
 *		   for alle de andre posisjonene. La for eksempel M best av 3 regioner og S best av 4
 *		   regioner. Dersom vektorverdien i indeks 5 da har verdien 1, betyr dette at region nr 2 i
 *		   M er mappet med region nr 2 i S.
 */
int * Sort::calculateBestCorrespondence(int *input, int width, int height)
{

	//Vektor som holder oversikt over ledige kolonner (regioner i S)
	bool *col = new bool[width];

	//Dynamisk vektor som holder oversikt over ledige rader (regioner i M)
	vector<int> row;

	//Vektorlengde (antall regioner i S multiplisert med antall regioner i M)
	int length = width*height;

	//Vektor som radvis holder sorterte kolonner (hver region i M assosieres med
	///sortert rekkeflge fra mest til minst like regioner i S)
	int *sortedRows = new int[length];

	//Vektor som holder sorterte verdier av inputvektor (differansen av to vektede
	//summer av den lille egenskapsmengden, der disse anvendes p en region fra M,
	//og en region fra S)
	int *array = new int[length];

	//Vektor som holder mapping av rader og kolonner (regioner fra M og S)
	int *output = new int[length];

	//Initialiserer vektorer med startverdier
	for (int i=0;i<length;i++) 
	{
		sortedRows[i] = i % width;
		array[i] = input[i];
		output[i] = 0;
		if ( i < width ) 
		{ 
			col[i] = true;
		}
	}

	//Sorterer radvis verdier i array og sortedRows, og legger aktuell radindeks inn i row-vektoren
	for (int i=0;i<height;i++) 
	{
		Quicksort<int>(array, sortedRows, i*width, (i+1)*width-1, length);
		row.push_back(i);
	}

	//Itererer s lenge det finnes en rad (region i M) som ikke er mappet med en kolonne (region i S)
	while ( row.size() > 0 )
	{
		//Oppretter variabler og startverdier
		int r_index=0, c=0, min=INFINITY_INT;
		int front_c;

		//G gjennom alle rader (regioner i M)
		for (unsigned int i=0;i<row.size();i++)
		{
			//La temp_p betegne aktuell rad (region i M)
			int temp_r = row.at(i);
			
			//La front_c betegne den kolonnen (regionen i S) som likner raden (regionen i M) mest
			front_c = getFrontValue(sortedRows, col, temp_r, width);

			//Dersom aktuell mapping gitt av raden temp_r og kolonnen front_c er den beste s langt
			if ( input[ temp_r * width + front_c ] < min )
			{
				//Sett variabelen min til denne mappingens verdi
				min = input[ temp_r * width + front_c ];

				//La r_index betegne aktuell rad (region i M)
				r_index = i;

				//La c betegne aktuell kolonne (region i S)
				c = front_c;
			}
		}

		//Angi kolonne som mappet region fra S i kolonnelisten col
		col[c] = false;

		//Merk av mapping p aktuell posisjon i outputvektor
		output[ row.at(r_index)*width + c ] = 1;

		//Fjern rad fra listen over umappede regioner i S
		row.erase( row.begin() + r_index );
	}

	//Utfrer annnenrangs mapping p output-vektoren og returnerer resultatet
	return secondOrderMapping(input, col, width, height, output);

}