#pragma once

///Konstant som representerer uendelig
const int INFINITY_INT = 1000000;


/**
 * @class Quicksort
 * @brief En "in-place"-implementasjon av quicksort.
 *
 * Denne templaten er en "in-place"-implementasjon av quicksort. Mer informasjon 
 * om quicksort algoritmen gis blant annet p Wikipedia: 
 * http://en.wikipedia.org/wiki/Quicksort
 *
 * Template-klassens kildekode er hentet fra:
 * http://users.sdsc.edu/~decastro/home/projects/datastruct/quicksort.cpp
 *
 * Undertegnede har utvidet klassen til  behandle en ekstra parameter (b),
 * som representerer regionenes opprinnelige kolonneposisjon i inputvektoren (a).
 * Dette gjres for  holde styr p regionenes "identifikasjon", siden 
 * "in-place"-sorteringen flytter p verdiene i inputvektoren a. Verdier flyttes 
 * helt identisk innad i vektor a og b.
 *
 * @param a Peker til vektor som inneholder de verdiene som skal sorteres.
 * @param b Peker til vektor som inneholder kolonneposisjonene til 
 *			regionene som sorteres.
 * @param l Nedre indeksgrense for hvor sortering skal finne sted. 
 * @param r vre indeksgrense for hvor sortering skal finne sted.
 * @param s Lengde p inputvektorene a og b.
 *
 * @version 1.0
 * @author Alex DeCastro, http://users.sdsc.edu/~decastro/home/projects/datastruct/quicksort.cpp
 * @author Modifisert og tilpasset av Per Gunnar Brdsen
 *
 */
template<class T>
void Quicksort(T *a, T *b, int l, int r, int s);


/**
 * @brief Metode som etter grdighetsprinsippet returnerer beste mapping fra en
 *		  region i et bilde til en av flere mulige regioner i bildeparets andre bilde.
 *
 * Parameterforklaring:
 * La (M,S) vre et bildepar, der M og S henholdsvis betegner bildet med minst og strst
 * antall regioner.
 *
 * @param sortedRows Peker til vektor som inneholder ferdigsorterte/prioriterte
 *		             mapperelasjoner mellom regioner i M og S.
 * @param col Peker til vektor med boolske verdier, som angir om aktuelle regioner 
			  i S er mappet fra fr av eller ikke.
 * @param row Betegner aktuell region i M.
 * @param width Antall regioner i S.
 *
 * @return Returnerer etter grdighetsprinsippet den beste mappingen fra en region i M
 *		   til en av flere mulige regioner i S.
 */
int getFrontValue(int *sortedRows, bool *col, int row, int width);


/**
 * @brief Metode som beregner annenrangs mappinger.
 *
 * Metode som beregner annenrangs mappinger i tilfeller nr bildene i bildeparet ikke har 
 * samme antall regioner. La (M,S) vre et bildepar, der M og S henholdsvis betegner bildet 
 * med minst (n stk) og strst (m stk) antall regioner. I tilfeller der m>n brukes denne 
 * metoden til  mappe m-n stk regioner, slik at alle regioner i S blir mappet en gang. En 
 * region i M kan da mappes til mer enn en region i S. Denne metoden brukes av 
 * calculateBestCorrespondence(...)-metoden.
 *
 * For mer informasjon se prosjektrapport kapittel 6.1.2 og 6.1.3:
 * http://www.idi.ntnu.no/~pergunb/rapport.pdf
 *
 * @param input Peker til vektor som representerer sammenlikningsgrunnlaget. Vektorens 
 *			    strrelse er da n*m. Alle regioner i M er sammenliknet mot alle regioner i
 *				S, og hver slik sammenlikning har sin posisjon i vektoren.
 * @param col Peker til vektor med boolske verdier som angir om kolonnen med aktuell indeks
 *		      er mappet eller ikke.
 * @param width Antall regioner i M.
 * @param height Antall regioner i S.
 * @param output Peker til vektor som holder den beste korresponansen/mappin mellom regioner i
 *				 i bildeparet. Denne vektoren vil ha verdien 1 i de posisjoner der to regioner
 *				 mappes, og verdien 0 for alle de andre posisjonene. La for eksempel M best 
 *				 av 3 regioner og S best av 4 regioner. Dersom vektorverdien i indeks 5 da har 
 *				 verdien 1, betyr dette at region nr 2 i M er mappet med region nr 2 i S.
 *
 * @return Peker til output-vektoren etter at metoden har oppdatert denne med annenrangs 
 *		   mappinger.
 */
int * secondOrderMapping(int *input, bool *col, int width, int height, int *output);


/**
 * @class Sort
 * @brief Beregner beste korrespondans/mapping mellom regioner i et bildepar.
 *
 * Klasse som beregner den beste korrespondansen/mappingen mellom regioner i et bildepar.
 * Beregningen gjres p bakgrunn av et sammenlikningsgrunnlag, der differansen av ulike 
 * regioners beregninger av den lille egenskapsmengden (areal, massesenter og omsluttende 
 * rektangel) mles. Algortimen anvender quicksort og flger en grdighetsstrategi, som
 * approksimerer den beste lsningen.
 *
 * For mer informasjon se prosjektrapport kapittel 6.1.2 (pseudo-kode gitt i Algoritme 4):
 * http://www.idi.ntnu.no/~pergunb/rapport.pdf. Implementasjonen avviker noe prosjektrapporten
 * ved at den ikke anvender en lenket liste struktur, og at radene bare ineholder kolonneverdi
 * (bruker i stedet en egen vektor til  holde styr p kolonneindekser). Hver rad inkluderer 
 * heller ikke radens posisjon som frste element.
 *
 * @version 1.0
 * @author Per Gunnar Brdsen
 */
class Sort
{

public:

	/**
	 * @brief Konstruktor
	 */
	Sort();


	/**
	 * @brief Destruktor
	 */
	~Sort();


	/**
	 * @brief Beregner beste korrespondans/mapping mellom regioner i et bildepar.
	 *
	 * Metode som beregner den beste korrespondanse/mapping mellom regioner i et bildepar. 
	 * Beregningen gjres p bakgrunn av et sammenlikningsgrunnlag, der differansen av ulike 
	 * regioners beregninger av den lille egenskapsmengden (areal, massesenter og omsluttende 
	 * rektangel) mles.
	 *
	 * @param input Vektor som representerer sammenlikningsgrunnlaget. La (M,S) vre et bildepar,
	 * der M og S henholdsvis betegner bildet med minst (n stk) og strst (m stk) antall regioner.
	 * Vektorens strrelse er da lik antall regioner i M multiplisert med antall regioner i S (n*m). 
	 * Alle regioner i M er sammenliknet mot alle regioner i S, og hver slik sammenlikning har sin 
	 * posisjon i vektoren.
	 *
	 * @param input Peker til vektor som inneholder sammenlikningsgrunnlaget for mapping av regioner i bildeparet.
	 * @param width Antall regioner i S (m stk).
	 * @param height Antall regioner i M (s stk).
	 *
	 * @return En peker til en vektor som holder den beste korrespondansen mellom regioner i bildeparet.
	 *		   Denne vektoren vil ha verdien 1 i de posisjoner der to regioner mappes, og verdien 0
	 *		   for alle de andre posisjonene. La for eksempel M best av 3 regioner og S best av 4
	 *		   regioner. Dersom vektorverdien i indeks 5 da har verdien 1, betyr dette at region nr 2 i
	 *		   M er mappet med region nr 2 i S.
	 */
	int * calculateBestCorrespondence(int *input, int width, int height);

};