/********************************************************************
   The Image Processing Library 98 - IPL98
   Copyright (C) by Ren Dencker Eriksen - edr@mip.sdu.dk

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation. Only
   addition is, that if the library is used in proprietary software
   it must be stated that IPL98 was used.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   More details can be found in the file licence.txt distributed
   with this library.
*********************************************************************/

#include "graphics.h"
#include "../ipl98_general_functions.h"
#include "../../kernel_c/algorithms/kernel_graphics.h"

namespace ipl{

CGraphics::CGraphics() : m_TotalFonts(3)
{
	m_FontColor=0;
	m_FontTransparent=true;
	m_FontActive=CourierBold13x18;
	m_FontAlignment=TOPLEFT;
	m_FontData.resize(m_TotalFonts);
	for(int i=0;i<m_TotalFonts;i++)
		m_FontData[i]=NULL;
}

CGraphics::CGraphics(UINT32 FontColor, bool FontTransparent, FONTALIGNMENT FontAlignment,
					 BITMAPFONT FontActive) : m_TotalFonts(3)
{
	m_FontColor=FontColor;
	m_FontTransparent=FontTransparent;
	m_FontActive=FontActive;
	m_FontAlignment=FontAlignment;
	m_FontData.resize(m_TotalFonts);
	for(int i=0;i<m_TotalFonts;i++)
		m_FontData[i]=NULL;
}

CGraphics::~CGraphics()
{
	for(int i=0;i<m_TotalFonts;i++)
	{
		if (m_FontData[i]!=NULL)
			free(m_FontData[i]);
	}
}

bool CGraphics::Line(const CPoint2D<INT16>& Start, const CPoint2D<INT16>& End, CPixelGroup& PixelGroup)
{
	T2DInt Start1, End1;
	Start1.x=Start.GetX();
	Start1.y=Start.GetY();
	End1.x=End.GetX();
	End1.y=End.GetY();
	if (k_GraphicsGenerateLine(Start1,End1, PixelGroup.GetTPixelGroupPtr())==false)
	{
		ostringstream ost;
		ost << "CGraphics::Line() Failed generating line points" << IPLAddFileAndLine;
		CError::ShowMessage(IPL_ERROR,ost);
		return false;
	}
	return true;
}

bool CGraphics::Line(const CPoint2D<INT16>& Start, const CPoint2D<INT16>& End, unsigned int Width,
					CPixelGroup& PixelGroup)
{
	T2DInt Start1, End1;
	Start1.x=Start.GetX();
	Start1.y=Start.GetY();
	End1.x=End.GetX();
	End1.y=End.GetY();
	if (k_GraphicsGenerateLineThick(Start1,End1,Width,PixelGroup.GetTPixelGroupPtr())==false)
	{
		ostringstream ost;
		ost << "CGraphics::Line() Failed generating line points" << IPLAddFileAndLine;
		CError::ShowMessage(IPL_ERROR,ost);
		return false;
	}
	return true;
}

bool CGraphics::Circle(const CPoint2D<INT16>& Center, unsigned int Radius, CPixelGroup& PixelGroup)
{
	T2DInt Center1;
	Center1.x=Center.GetX();
	Center1.y=Center.GetY();
	if (k_GraphicsGenerateCircle(Center1,Radius,PixelGroup.GetTPixelGroupPtr())==false)
	{
		ostringstream ost;
		ost << "CGraphics::Circle() Failed generating circle points" << IPLAddFileAndLine;
		CError::ShowMessage(IPL_ERROR,ost);
		return false;
	}
	return true;
}

bool CGraphics::Circle(const CPoint2D<INT16>& Center, unsigned int Radius1, unsigned int Radius2,
					   CPixelGroup& PixelGroup)
{
	T2DInt Center1;
	Center1.x=Center.GetX();
	Center1.y=Center.GetY();
	if (k_GraphicsGenerateCircleRing(Center1,Radius1,Radius2,PixelGroup.GetTPixelGroupPtr())==false)
	{
		ostringstream ost;
		ost << "CGraphics::Circle() Failed generating circle points" << IPLAddFileAndLine;
		CError::ShowMessage(IPL_ERROR,ost);
		return false;
	}
	return true;
}

bool CGraphics::CircleW(const CPoint2D<INT16>& Center, unsigned int Radius, unsigned int Width,
					CPixelGroup& PixelGroup)
{
	T2DInt Center1;
	Center1.x=Center.GetX();
	Center1.y=Center.GetY();
	if (k_GraphicsGenerateCircleRingW(Center1,Radius,Width,PixelGroup.GetTPixelGroupPtr())==false)
	{
		ostringstream ost;
		ost << "CGraphics::CircleW() Failed generating circle points" << IPLAddFileAndLine;
		CError::ShowMessage(IPL_ERROR,ost);
		return false;
	}
	return true;
}

bool CGraphics::Ellipse(const CPoint2D<INT16>& Center, unsigned int RadiusX, 
										unsigned int RadiusY, CPixelGroup& PixelGroup)
{
	T2DInt Center1;
	Center1.x=Center.GetX();
	Center1.y=Center.GetY();
	if (k_GraphicsGenerateEllipse(Center1,RadiusX,RadiusY,PixelGroup.GetTPixelGroupPtr())==false)
	{
		ostringstream ost;
		ost << "CGraphics::Circle() Failed generating ellipse points" << IPLAddFileAndLine;
		CError::ShowMessage(IPL_ERROR,ost);
		return false;
	}
	return true;
}

bool CGraphics::Cross(const CPoint2D<INT16>& Center, unsigned int Size, CPixelGroup& PixelGroup)
{
	T2DInt Center1;
	Center1.x=Center.GetX();
	Center1.y=Center.GetY();
	if (k_GraphicsGenerateCross(Center1,Size,PixelGroup.GetTPixelGroupPtr())==false)
	{
		ostringstream ost;
		ost << "CGraphics::Cross() Failed generating cross points" << IPLAddFileAndLine;
		CError::ShowMessage(IPL_ERROR,ost);
		return false;
	}
	return true;
}

bool CGraphics::Plus(const CPoint2D<INT16>& Center, unsigned int Size, CPixelGroup& PixelGroup)
{
	T2DInt Center1;
	Center1.x=Center.GetX();
	Center1.y=Center.GetY();
	if (k_GraphicsGeneratePlus(Center1,Size,PixelGroup.GetTPixelGroupPtr())==false)
	{
		ostringstream ost;
		ost << "CGraphics::Plus() Failed generating plus points" << IPLAddFileAndLine;
		CError::ShowMessage(IPL_ERROR,ost);
		return false;
	}
	return true;
}

bool CGraphics::Rectangle(const CPoint2D<INT16>& TopLeft, const CPoint2D<INT16>& BottomRight, 
						CPixelGroup& PixelGroup)
{
	T2DInt TopLeft1;
	TopLeft1.x=TopLeft.GetX();
	TopLeft1.y=TopLeft.GetY();
	T2DInt BottomRight1;
	BottomRight1.x=BottomRight.GetX();
	BottomRight1.y=BottomRight.GetY();
	if (k_GraphicsGenerateRectangle(TopLeft1,BottomRight1,PixelGroup.GetTPixelGroupPtr())==false)
	{
		ostringstream ost;
		ost << "CGraphics::Rectangle() Failed generating rectangle points" << IPLAddFileAndLine;
		CError::ShowMessage(IPL_ERROR,ost);
		return false;
	}
	return true;
}

void CGraphics::SetFontActive(BITMAPFONT Font)
{
	m_FontActive=Font;
}

CGraphics::BITMAPFONT CGraphics::GetFontActive() const
{
	return m_FontActive;
}

void CGraphics::SetFontTransparent(bool Transparent)
{
	m_FontTransparent=Transparent;
}

bool CGraphics::GetFontTransparent() const
{
	return m_FontTransparent;
}

void CGraphics::SetFontColor(UINT32 Color)
{
	m_FontColor=Color;
}

UINT32 CGraphics::GetFontColor() const
{
	return m_FontColor;
}

void CGraphics::SetFontBackgroundColor(UINT32 Color)
{
	m_FontBackgroundColor=Color;
}

UINT32 CGraphics::GetFontBackgroundColor() const
{
	return m_FontBackgroundColor;
}

void CGraphics::SetFontAlignment(FONTALIGNMENT Align)
{
	m_FontAlignment=Align;
}

CGraphics::FONTALIGNMENT CGraphics::GetFontAlignment() const
{
	return m_FontAlignment;
}

bool CGraphics::PlotString(const char* str, const CPoint2D<int>& Pos, CStdImage& Img)
{
	return PlotString(str,Pos.GetX(),Pos.GetY(),Img);
}

bool CGraphics::PlotString(const char* str, int x, int y, CStdImage& Img)
{
	if (m_FontData[m_FontActive]==NULL)
	{
		InitFont();
	}
	UINT8* pFontData;
	UINT8 ASCIIStart;
	int TotalChars,CharWidth,CharHeight;
	GetFontInfo(m_FontActive,&pFontData,ASCIIStart,TotalChars,CharWidth,CharHeight);
	CPoint2D<int> P(x,y);
	// adjust P according to alignent value
	if (AdjustAlignment(str,P)==false)
		return false;
	int Size=GetSizeString(str);
	int Count=0;
	while(Count<Size)
	{
		if (PlotChar(*(str+Count),P,Img)==false)
			return false;
		P += CPoint2D<int>(CharWidth,0);
		Count++;
	}
	return true;
}

bool CGraphics::GetFontInfo(BITMAPFONT Font,UINT8** ppFontData, UINT8& ASCIIStart, int& TotalChars,int& CharWidth,int& CharHeight)
{
	*ppFontData=m_FontData[m_FontActive];
	switch(Font)
	{
	case CourierBold13x18:
		{
			ASCIIStart=31;
			TotalChars=256;
			CharWidth=13;
			CharHeight=18;
			break;
		}
	case Terminal12x16:
		{
			ASCIIStart=0;
			TotalChars=256;
			CharWidth=12;
			CharHeight=16;
			break;
		}
	case Courier8x12:
		{
			ASCIIStart=0;
			TotalChars=256;
			CharWidth=8;
			CharHeight=12;
			break;
		}
	default:
		{
			ostringstream ost;
			ost << "CGraphics::PlotChar() Unknown font is active" << IPLAddFileAndLine;
			CError::ShowMessage(IPL_ERROR,ost);
			return false;
		}
	}
	return true;
}

////////////////////////////////////////////////////
////////////////////////////////////////////////////
//////////    private methods    ///////////////////
////////////////////////////////////////////////////
////////////////////////////////////////////////////

bool CGraphics::AdjustAlignment(const char* str, CPoint2D<int>& Pos)
{
	UINT8* pFontData;
	UINT8 ASCIIStart;
	int TotalChars,CharWidth,CharHeight;
	GetFontInfo(m_FontActive,&pFontData,ASCIIStart,TotalChars,CharWidth,CharHeight);
	switch(m_FontAlignment)
	{
	case TOPLEFT:
		{
			// do nothing
			break;
		}
	case BOTTOMLEFT:
		{
			Pos.SetY(Pos.GetY()-CharHeight);
			break;
		}
	case TOPMID:
		{
			int Size=GetSizeString(str);
			Pos.SetX(Pos.GetX()-Round(Size*CharWidth/2));
			break;
		}
	case BOTTOMMID:
		{
			int Size=GetSizeString(str);
			Pos.SetX(Pos.GetX()-Round(Size*CharWidth/2));
			Pos.SetY(Pos.GetY()-CharHeight);
			break;
		}
	case TOPRIGHT:
		{
			int Size=GetSizeString(str);
			Pos.SetX(Pos.GetX()-Round(Size*CharWidth));
			break;
		}
	case BOTTOMRIGHT:
		{
			int Size=GetSizeString(str);
			Pos.SetX(Pos.GetX()-Round(Size*CharWidth));
			Pos.SetY(Pos.GetY()-CharHeight);
			break;
		}
	default:
		{
			ostringstream ost;
			ost << "CGraphics::AdjustAlignment() Unknown alignment value" << IPLAddFileAndLine;
			CError::ShowMessage(IPL_ERROR,ost);
			return false;
		}
	}
	return true;
}

int CGraphics::GetSizeString(const char* str)
{
	int Size=0;
	const char* s=str;
	while(*s!='\0')
	{
		Size++;
		s++;
	}
	return Size;
}


bool CGraphics::PlotChar(char ch, const CPoint2D<int>& Pos, CStdImage& Img)
{
	int TotalChars;
	int CharWidth;
	int CharHeight;
	UINT8 ASCIIStart;
	UINT8 c=(UINT8)ch;
	UINT8* pFontData=NULL;
	int MinX=Img.GetMinX();
	int MaxX=Img.GetMaxX();
	int MinY=Img.GetMinY();
	int MaxY=Img.GetMaxY();
	if (GetFontInfo(m_FontActive,&pFontData,ASCIIStart,TotalChars,CharWidth,CharHeight)==false)
		return false;
	int x,y,k,Index;
	if ((c<ASCIIStart) || (c>=TotalChars)) c=ASCIIStart; // if character not used, print the lowest available
	c-=ASCIIStart;
	int ScanLength=(CharWidth*(TotalChars-ASCIIStart))/8+1; // in bytes
	int Offset=CharWidth*(int)c;
	int StartIndex=(int)Offset/8;
	int BitOffsetF=Offset % 8;
	// number of whole bytes in width of character to be read after the first part of a byte has been read
	int WholeBytes=(CharWidth-(8-BitOffsetF))/8; 
	int RemainingBits=CharWidth-((8-BitOffsetF)+8*WholeBytes); // 5

	for(y=0; y<CharHeight; y++)
	{
		x=Pos.GetX();
		Index=y*ScanLength+StartIndex;
		// run in part of first byte
		for(k=0;k<8-BitOffsetF;k++)
		{
			UINT8 PlotVal=pFontData[Index];
			PlotVal <<= BitOffsetF + k;
			PlotVal >>= 7;
			if (PlotVal==0)
			{
				if ((x>MinX) && (x<MaxX) && (Pos.GetY()+y>MinY) && (Pos.GetY()+y<MaxY))
					Img.SetPixelFast(x,Pos.GetY()+y,m_FontColor);
			}
			else if (m_FontTransparent==false)
			{
				if ((x>MinX) && (x<MaxX) && (Pos.GetY()+y>MinY) && (Pos.GetY()+y<MaxY))
					Img.SetPixelFast(x,Pos.GetY()+y,m_FontBackgroundColor);
			}
			x++;
		}
		// run in whole bytes inbetween
		for(int i=0; i<WholeBytes; i++)
		{
			Index++;
			for(k=0; k<8; k++)
			{
				UINT8 PlotVal=pFontData[Index];
				PlotVal <<= k;
				PlotVal >>= 7;
				if (PlotVal==0)
				{
					if ((x>MinX) && (x<MaxX) && (Pos.GetY()+y>MinY) && (Pos.GetY()+y<MaxY))
						Img.SetPixelFast(x,Pos.GetY()+y,m_FontColor);
				}
				else if (m_FontTransparent==false)
				{
					if ((x>MinX) && (x<MaxX) && (Pos.GetY()+y>MinY) && (Pos.GetY()+y<MaxY))
						Img.SetPixelFast(x,Pos.GetY()+y,m_FontBackgroundColor);
				}
				x++;
			}
		}
		// run in part of last byte
		Index++;
		for(k=0;k<RemainingBits;k++)
		{
			UINT8 PlotVal=pFontData[Index];
			PlotVal <<= k;
			PlotVal >>= 7;
			if (PlotVal==0)
			{
				if ((x>MinX) && (x<MaxX) && (Pos.GetY()+y>MinY) && (Pos.GetY()+y<MaxY))
					Img.SetPixelFast(x,Pos.GetY()+y,m_FontColor);
			}
			else if (m_FontTransparent==false)
			{
				if ((x>MinX) && (x<MaxX) && (Pos.GetY()+y>MinY) && (Pos.GetY()+y<MaxY))
					Img.SetPixelFast(x,Pos.GetY()+y,m_FontBackgroundColor);
			}
			x++;
		}
	}
	return true;
}


bool CGraphics::InitFont()
{
	if (m_FontData[m_FontActive]!=NULL)
	{
		// allready initialize - do nothing
		return true;
	}
	switch(m_FontActive)
	{
	case CourierBold13x18:
		{
			if (m_FontData[m_FontActive]==NULL)
				InitCourierBold13x18();
			break;
		}
	case Courier8x12:
		{
			if (m_FontData[m_FontActive]==NULL)
				InitCourier8x12();
			break;
		}
	case Terminal12x16:
		{
			if (m_FontData[m_FontActive]==NULL)
				InitTerminal12x16();
			break;
		}
	default:
		{
			ostringstream ost;
			ost << "CGraphics::InitFont() Unknown font is active" << IPLAddFileAndLine;
			CError::ShowMessage(IPL_ERROR,ost);
			return false;
		}
	}
	return true;
}

void CGraphics::InitCourierBold13x18()
{
	UINT8 Data1[]={255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,207,255,159,240,254,67,243,159,225,255,255,255,255,207,255,159,240,255,51,
		252,255,249,255,15,243,63,255,249,15,243,255,231,252,63,144,254,103,255,255,255,254,127,252,255,135,249,159,243,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,248, /*end of scanline 0 */
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,7,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,231,255,63,230,127,255,255,255,243,255,255,255,255,231,255,63,230,127,255,
		254,127,243,254,103,255,255,255,255,255,249,255,207,249,159,255,255,255,255,255,255,255,63,249,255,51,255,255,243,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,248, /*end of scanline 1 */
		255,255,255,255,255,255,255,255,254,127,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,159,255,255,207,255,255,143,255,255,255,143,255,255,195,255,254,63,255,63,252,252,127,248,127,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,252,57,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,153,255,255,224,127,255,255,255,255,255,255,255,255,195,255,255,240,255,135,255,63,255,255,255,255,255,255,255,63,224,127,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,159,255,207,249,255,9,249,159,243,255,255,255,255,159,255,207,249,255,51,249,255,249,255,31,242,127,131,252,39,231,255,243,254,127,194,126,103,
		255,255,255,252,255,254,127,207,249,159,249,248,255,243,56, /*end of scanline 2 */
		240,63,255,255,63,230,127,153,254,127,15,254,15,252,255,243,254,127,249,255,255,255,255,255,255,255,255,255,195,254,63,240,255,7,255,159,128,255,195,224,31,195,254,31,255,255,255,255,255,255,255,255,240,63,129,252,63,0,127,128,64,30,0,56,0,240,12,60,56,7,240,
		24,96,192,126,63,17,224,240,60,1,252,15,0,127,0,192,2,30,16,240,143,196,24,33,225,128,31,131,255,255,193,255,15,255,255,231,255,255,207,255,255,255,207,255,255,159,255,255,63,255,63,252,254,127,254,127,255,255,255,255,255,255,255,255,255,255,255,252,255,255,
		255,255,255,255,255,255,255,255,255,227,254,127,199,249,153,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,255,159,249,255,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,255,
		255,63,249,255,7,255,254,30,31,159,240,31,153,248,7,255,63,255,255,255,255,252,3,255,255,153,255,255,230,127,51,254,127,255,255,1,255,255,255,252,63,207,63,255,255,255,255,255,255,252,255,135,252,63,225,255,15,248,127,195,252,0,248,4,0,96,3,0,24,0,
		240,15,128,124,3,224,28,1,227,193,224,127,3,248,31,192,254,7,255,255,129,33,225,15,8,120,67,194,30,16,255,248,127,207,255,159,240,254,67,249,159,225,255,255,255,255,207,255,159,240,255,51,252,255,243,254,79,242,127,207,249,15,243,255,231,252,63,144,254,103,
		255,255,255,254,127,252,255,135,249,159,243,252,255,243,56, /*end of scanline 3 */
		240,63,255,255,63,230,127,153,248,14,103,60,231,252,255,231,255,63,201,63,255,255,255,255,255,255,255,231,153,248,63,230,126,115,255,31,159,255,31,231,159,153,252,207,255,255,255,255,255,255,255,255,231,159,60,255,63,207,62,60,115,199,159,62,124,199,142,126,127,63,254,
		124,243,243,255,30,56,249,199,143,60,241,227,207,62,120,206,115,63,57,249,159,230,126,115,243,159,159,159,207,255,249,254,103,255,255,243,255,255,207,255,255,255,207,255,255,159,255,255,63,255,255,255,254,127,254,127,255,255,255,255,255,255,255,255,255,255,255,252,255,255,
		255,255,255,255,255,255,255,255,255,207,254,127,243,249,195,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,255,63,249,255,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,255,
		255,63,249,254,115,255,255,63,63,159,231,159,255,243,243,224,63,255,255,255,255,249,249,255,255,153,255,255,254,127,199,252,255,255,252,7,255,255,255,255,63,207,63,255,243,243,159,152,124,252,255,231,255,63,249,255,207,254,127,243,255,28,227,199,62,121,243,207,158,124,
		254,127,243,255,159,252,255,60,113,243,143,28,120,227,199,30,56,241,255,254,60,115,243,159,156,252,231,231,63,60,255,243,63,231,255,63,230,127,255,255,255,243,255,255,255,255,231,255,63,230,127,255,254,127,231,252,231,255,255,7,255,255,249,255,207,249,159,255,255,255,
		255,255,255,255,63,249,255,51,255,255,255,252,255,255,248, /*end of scanline 4 */
		240,63,255,255,63,230,127,153,243,142,102,124,255,252,255,231,255,63,224,127,207,255,255,255,255,255,255,207,60,255,63,207,63,249,255,31,159,254,127,231,159,60,249,231,255,255,255,255,159,255,254,127,231,158,126,127,63,207,158,126,115,231,159,62,124,207,206,126,127,63,254,
		124,231,243,255,30,56,249,207,207,62,115,243,207,156,252,206,115,63,57,249,159,231,60,243,243,159,63,159,231,255,249,255,255,255,255,255,255,255,207,255,255,255,207,255,255,159,255,255,63,255,255,255,254,127,254,127,255,255,255,255,255,255,255,255,255,255,255,252,255,255,
		255,255,255,255,255,255,255,255,255,207,254,127,243,255,255,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,255,63,243,255,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,255,
		255,255,224,62,127,243,207,63,63,159,231,159,255,231,249,207,63,255,255,255,255,243,252,255,255,153,255,255,240,255,243,255,255,255,252,7,255,255,255,255,63,207,63,255,195,230,31,51,57,255,255,231,255,63,249,255,207,254,127,243,255,28,231,231,62,121,243,207,158,124,
		254,127,243,255,159,252,255,62,113,243,159,156,252,231,231,63,57,249,207,206,124,115,243,159,156,252,231,231,63,60,255,243,159,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,243,255,255,255,255,255,255,255,255,255,255,
		252,255,255,255,255,255,255,255,255,255,255,252,255,255,248, /*end of scanline 5 */
		240,63,255,255,63,230,126,0,243,206,100,252,255,252,255,207,255,159,240,255,207,255,255,255,255,255,255,159,60,255,63,255,63,249,254,31,159,254,127,255,63,60,249,231,249,255,207,255,63,255,255,63,255,158,126,126,31,207,156,255,243,243,159,254,127,159,254,126,127,63,254,
		124,207,243,255,12,56,121,159,231,62,103,249,207,156,255,254,127,63,60,243,156,231,153,249,231,254,127,159,243,255,249,255,255,255,255,255,252,15,192,127,128,248,15,192,254,3,240,15,1,252,63,192,254,96,254,126,4,120,7,240,62,1,252,3,128,63,0,240,15,28,
		112,240,135,132,24,33,225,128,31,207,254,127,243,255,255,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,255,255,255,255,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,255,
		255,255,206,62,127,248,30,30,31,159,243,255,255,230,9,224,63,255,255,255,255,242,12,255,255,195,255,63,231,255,51,255,255,158,120,7,255,255,255,255,63,224,127,255,243,207,158,124,115,255,255,195,254,31,240,255,135,252,63,225,254,31,207,255,63,249,255,207,254,127,
		254,127,243,255,159,252,255,63,48,243,63,201,254,79,242,127,147,252,231,156,249,51,243,159,156,252,231,231,158,124,7,243,159,129,252,15,224,127,3,248,31,192,248,1,248,15,129,252,15,224,127,3,248,127,195,254,31,240,255,129,240,15,224,127,3,248,31,192,254,7,
		252,255,128,113,199,142,60,113,227,142,30,28,7,135,128 /*end of scanline 6 */
	};
	UINT8 Data2[]={240,63,255,255,63,230,127,51,243,255,9,254,127,252,255,207,255,159,240,255,207,255,255,255,255,255,255,63,60,255,63,255,63,243,252,159,129,252,255,255,63,153,249,231,249,255,207,254,127,0,63,159,255,62,96,126,31,207,60,255,243,243,156,254,115,159,254,126,127,63,254,
		124,159,243,255,12,57,57,159,231,62,103,249,207,158,127,254,127,63,60,243,156,231,195,252,207,252,255,159,249,255,249,255,255,255,255,255,249,231,199,63,56,243,143,158,127,159,231,31,28,255,63,252,254,115,254,127,17,60,115,231,159,28,249,199,227,158,120,252,255,158,
		121,249,207,207,60,243,243,159,63,207,254,127,243,255,255,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,255,255,255,255,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,255,
		255,63,159,63,63,243,207,204,255,159,225,255,255,228,225,255,255,255,255,255,255,242,100,255,255,255,255,63,224,127,135,255,255,158,120,7,255,255,255,252,15,255,255,255,243,159,156,255,39,252,255,195,254,31,240,255,135,252,63,225,254,19,207,255,57,249,207,206,126,115,
		254,127,243,255,159,252,255,63,50,115,63,201,254,79,242,127,147,252,243,60,243,51,243,159,156,252,231,231,204,252,243,243,159,60,249,231,207,62,121,243,207,158,115,156,243,143,60,249,231,207,62,121,254,127,243,255,159,252,255,60,248,231,207,62,121,243,207,158,124,243,
		255,255,60,121,231,207,62,121,243,207,63,60,113,207,200, /*end of scanline 7 */
		240,63,255,255,63,255,255,51,249,255,243,252,63,255,255,207,255,159,230,127,207,255,255,255,255,255,254,127,60,255,63,254,127,199,252,159,156,252,15,254,127,195,252,199,255,255,255,252,255,255,255,207,254,126,78,124,207,192,60,255,243,243,128,254,3,159,254,0,127,63,254,
		124,63,243,255,33,57,153,159,231,60,231,249,207,63,15,254,127,63,62,103,200,79,231,254,31,249,255,159,252,255,249,255,255,255,255,255,255,231,207,158,124,231,207,63,63,159,207,159,60,255,63,252,254,103,254,127,51,60,243,207,207,62,115,231,231,252,252,252,255,158,
		124,243,204,207,153,249,231,158,127,207,254,127,243,255,255,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,255,255,255,255,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,255,
		255,63,159,254,7,243,207,225,255,159,228,255,255,228,249,192,63,255,255,255,255,242,100,255,255,255,255,63,255,255,255,255,255,158,124,7,252,255,255,255,255,192,63,255,243,63,153,243,15,252,255,153,252,207,230,127,51,249,159,204,252,131,207,255,1,248,15,192,126,3,
		254,127,243,255,159,252,252,7,51,51,63,201,254,79,242,127,147,252,248,124,231,51,243,159,156,252,231,231,225,252,249,242,63,252,255,231,255,63,249,255,207,254,127,156,231,206,126,115,243,159,156,252,254,127,243,255,159,252,254,126,121,231,159,156,252,231,231,63,57,249,
		255,254,120,121,231,207,62,121,243,207,158,124,249,231,152, /*end of scanline 8 */
		240,63,255,255,63,255,255,51,252,63,243,252,159,255,255,207,255,159,255,252,0,255,255,255,255,255,252,255,60,255,63,252,255,243,249,159,254,124,103,254,127,153,254,7,255,255,255,249,255,255,255,231,252,254,78,124,207,207,156,255,243,243,156,254,115,158,6,126,127,63,254,
		124,31,243,255,33,57,153,159,231,1,231,249,192,127,227,254,127,63,62,103,200,79,231,255,63,249,255,159,254,127,249,255,255,255,255,255,254,7,207,158,127,231,207,63,63,159,207,159,60,255,63,252,254,79,254,127,51,60,243,207,207,62,115,231,231,254,127,252,255,158,
		124,243,204,207,195,249,231,252,255,207,254,127,243,255,255,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,255,255,255,255,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,255,
		255,63,159,255,63,243,206,0,31,159,230,127,255,228,249,255,255,153,240,15,255,242,100,255,255,255,248,7,255,255,255,255,255,158,124,7,252,255,255,255,255,255,254,103,192,102,0,56,25,249,255,153,252,207,230,127,51,249,159,204,252,147,207,255,57,249,207,206,126,115,
		254,127,243,255,159,252,255,63,51,51,63,201,254,79,242,127,147,252,252,252,231,51,243,159,156,252,231,231,225,252,249,243,159,192,254,7,240,63,129,252,15,224,124,28,231,254,126,115,243,159,156,252,254,127,243,255,159,252,254,126,121,231,159,156,252,231,231,63,57,249,
		192,14,114,121,231,207,62,121,243,207,158,124,249,231,152, /*end of scanline 9 */
		240,63,255,255,63,255,255,51,255,159,228,57,201,255,255,207,255,159,255,255,207,255,255,0,63,255,249,255,60,255,63,249,255,249,243,159,254,124,243,252,255,60,255,231,255,255,255,252,255,255,255,207,252,254,78,121,231,207,204,255,243,243,159,254,127,159,206,126,127,63,254,
		124,207,243,255,51,57,201,159,231,63,231,249,204,255,249,254,127,63,62,103,195,15,195,255,63,243,255,159,255,63,249,255,255,255,255,255,248,231,207,158,127,231,207,0,63,159,207,159,60,255,63,252,254,15,254,127,51,60,243,207,207,62,115,231,231,255,3,252,255,158,
		126,103,224,31,231,252,207,249,255,31,254,127,248,255,255,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,255,255,255,255,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,255,
		255,63,159,255,63,243,207,243,255,255,243,63,255,228,249,255,255,51,255,207,0,50,12,255,255,255,255,63,255,255,255,255,255,158,127,7,255,255,255,255,255,255,255,51,252,199,225,159,49,243,255,0,248,7,192,62,1,240,15,128,124,31,207,255,63,249,255,207,254,127,
		254,127,243,255,159,252,255,63,51,147,63,201,254,79,242,127,147,252,248,124,207,51,243,159,156,252,231,231,243,252,243,243,207,28,248,231,199,62,57,241,207,142,121,128,231,254,0,112,3,128,28,0,254,127,243,255,159,252,254,126,121,231,159,156,252,231,231,63,57,249,
		255,254,102,121,231,207,62,121,243,207,204,252,249,243,56, /*end of scanline 10 */
		240,63,255,255,255,255,252,1,255,207,201,153,225,255,255,207,255,159,255,255,207,255,255,255,255,255,243,255,60,255,63,243,255,249,240,15,254,124,243,252,255,60,255,207,255,255,255,254,127,0,63,159,255,254,78,120,7,207,204,255,243,243,159,254,127,159,206,126,127,63,254,
		124,231,243,255,51,57,225,159,231,63,231,249,206,127,252,254,127,63,63,15,195,15,153,255,63,231,255,159,255,159,249,255,255,255,255,255,243,231,207,158,127,231,207,63,255,159,207,159,60,255,63,252,254,39,254,127,51,60,243,207,207,62,115,231,231,255,249,252,255,158,
		126,103,224,31,195,252,207,243,255,207,254,127,243,255,255,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,255,255,255,255,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,255,
		255,63,159,255,63,248,31,243,255,255,249,159,255,228,225,255,254,103,255,207,255,242,76,255,255,255,255,63,255,255,255,255,255,158,127,135,255,255,255,255,255,255,255,153,249,135,207,158,97,231,255,60,249,231,207,62,121,243,207,158,121,159,207,255,63,249,255,207,254,127,
		254,127,243,255,159,252,255,63,51,195,63,201,254,79,242,127,147,252,243,60,159,51,243,159,156,252,231,231,243,252,7,243,206,124,243,231,159,60,249,231,207,62,115,159,231,254,127,243,255,159,252,255,254,127,243,255,159,252,254,126,121,231,159,156,252,231,231,63,57,249,
		255,254,78,121,231,207,62,121,243,207,204,252,249,243,56, /*end of scanline 11 */
		240,63,255,255,255,255,254,103,243,207,153,153,243,255,255,207,255,159,255,255,207,255,255,255,255,255,231,255,60,255,63,231,63,249,255,159,254,124,243,249,255,60,255,207,255,255,255,255,63,255,255,63,255,254,78,115,243,207,206,126,115,231,159,62,127,207,206,126,127,63,158,
		124,243,243,231,63,57,241,207,207,63,243,243,207,60,252,254,127,63,63,15,231,159,60,255,63,207,159,159,255,207,249,255,255,255,255,255,243,231,207,158,127,231,207,63,63,159,207,159,60,255,63,252,254,115,254,127,51,60,243,207,207,62,115,231,231,252,252,252,207,158,
		127,15,243,63,153,254,31,231,159,207,254,127,243,255,255,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,255,255,255,255,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,255,
		255,63,207,63,63,243,207,243,255,159,252,159,255,230,9,255,255,51,255,255,255,242,100,255,255,255,255,63,255,255,255,255,255,158,127,135,255,255,255,255,255,255,255,51,243,39,156,60,201,231,158,126,115,243,159,156,252,231,231,63,57,156,231,231,62,121,243,207,158,124,
		254,127,243,255,159,252,255,62,115,227,159,156,252,231,231,63,57,249,231,158,62,115,243,159,156,252,231,231,243,252,255,243,206,124,243,231,159,60,249,231,207,62,115,159,231,254,126,115,243,159,156,252,254,127,243,255,159,252,254,126,121,231,159,156,252,231,231,63,57,249,
		252,254,30,121,231,207,62,121,243,207,225,252,249,248,120, /*end of scanline 12 */
		240,63,255,255,63,255,254,103,241,207,57,153,225,255,255,207,255,159,255,255,207,254,127,255,255,159,207,255,153,255,63,207,62,115,255,159,156,254,103,249,255,153,255,31,249,255,207,255,159,255,254,127,252,254,96,115,243,207,158,60,115,199,159,62,127,199,142,126,127,63,158,
		124,249,243,231,63,57,241,199,143,63,241,227,207,156,121,254,127,158,127,159,231,158,126,127,63,159,159,159,255,231,249,255,255,255,255,255,243,231,199,63,60,243,143,158,127,159,231,31,60,255,63,252,254,121,254,127,51,60,243,231,159,28,249,199,231,252,121,252,207,156,
		127,15,243,63,60,254,31,207,159,207,254,127,243,255,255,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,255,255,255,255,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,255,
		255,63,224,126,127,255,255,243,255,159,254,31,255,231,249,255,255,153,255,255,255,243,252,255,255,255,255,255,255,255,255,255,255,140,127,135,255,255,255,255,255,255,254,103,231,3,57,249,192,231,158,126,115,243,159,156,252,231,231,63,57,156,227,199,62,121,243,207,158,124,
		254,127,243,255,159,252,255,60,115,227,143,28,120,227,199,30,56,241,207,206,60,121,231,207,62,121,243,207,243,252,255,243,158,124,243,231,159,60,249,231,207,62,115,156,243,207,60,249,231,207,62,121,254,127,243,255,159,252,255,60,249,231,207,62,121,243,207,158,124,243,
		252,254,60,249,199,206,62,113,243,143,225,252,113,248,120 /*end of scanline 13 */
	};
	UINT8 Data3[]={240,63,255,255,63,255,254,103,240,31,252,60,12,255,255,207,255,159,255,255,255,254,127,255,255,159,159,255,195,248,7,192,63,7,254,15,193,255,15,249,255,195,248,127,249,255,207,255,255,255,255,255,252,254,127,225,225,0,63,129,192,30,0,56,15,240,60,60,56,7,192,
		248,124,64,6,30,16,121,240,60,7,252,15,3,204,3,248,31,192,255,159,231,156,24,60,15,128,31,159,255,243,249,255,255,255,255,255,248,3,128,127,129,248,7,192,254,3,240,30,24,120,7,252,252,124,112,14,17,24,97,240,63,1,252,7,128,252,3,254,31,192,
		63,159,243,60,24,63,63,128,31,207,254,127,243,255,255,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,129,255,255,255,255,3,248,31,192,254,7,240,63,129,252,15,224,127,3,248,31,192,254,7,240,63,255,
		255,63,249,254,1,255,255,192,255,159,255,63,255,243,243,255,255,255,255,255,255,249,249,255,255,255,248,7,255,255,255,255,255,129,63,135,255,255,255,255,255,255,255,255,207,230,120,19,249,240,60,60,33,225,15,8,120,67,194,30,16,0,248,28,0,96,3,0,24,0,
		240,15,128,124,3,224,28,1,224,243,224,127,3,248,31,192,254,7,255,252,129,252,15,224,127,3,248,31,192,240,63,194,63,0,120,3,192,30,0,240,7,128,56,1,248,31,129,252,15,224,127,3,240,15,128,124,3,224,31,129,240,195,224,127,3,248,31,192,254,7,
		255,254,1,252,3,224,31,0,248,7,243,252,7,252,248, /*end of scanline 14 */
		255,255,255,255,255,255,255,255,254,127,255,255,255,255,255,231,255,63,255,255,255,252,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,159,255,255,255,255,255,255,255,63,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,252,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,159,255,255,249,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,159,255,255,255,252,255,255,255,255,255,255,255,255,255,63,255,231,255,255,255,255,255,255,
		255,255,255,255,255,255,63,255,255,207,254,127,243,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,249,255,255,255,255,255,255,159,231,159,255,248,7,255,255,255,255,255,255,252,3,255,255,255,255,255,255,255,255,255,255,159,255,135,255,255,231,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,254,127,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,254,127,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,243,252,255,252,248, /*end of scanline 15 */
		255,255,255,255,255,255,255,255,254,127,255,255,255,255,255,231,255,63,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,129,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,248,103,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,159,255,255,249,255,255,0,7,255,255,255,255,255,255,255,255,255,255,255,255,159,255,255,255,252,255,255,255,255,255,255,255,255,255,63,255,231,255,255,255,255,255,255,
		255,255,255,255,255,254,127,255,255,207,254,127,243,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,159,231,159,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,159,254,1,255,255,243,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,63,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,63,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,231,252,255,249,248, /*end of scanline 16 */
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,243,254,127,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,243,15,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,131,255,255,193,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,63,255,255,255,249,255,255,255,255,255,255,255,255,255,63,255,231,255,255,255,255,255,255,
		255,255,255,255,255,254,127,255,255,227,254,127,199,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,159,224,63,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,159,255,255,255,255,243,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,63,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,63,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,231,252,255,249,248 /*end of scanline 17 */
	};
	// following two variables must correspond to numbers of scanlines in each variable above, 1 and 2 must be the same
	int ScanLines1And2=7; // number of scanlines stored in each variable xxxxData1 and xxxxData2
	int ScanLines3=4; // number of scanlines stored in variable xxxxData3
	// following two variables can be found in output file written by the font data generator
	int TotalNumbers=7506; // total numbers defining font
	int NumbersPrScanLine=366; // numbers pr scanline
	
	UINT8* pFontData=(UINT8*)malloc(TotalNumbers);
	memcpy(pFontData,Data1,ScanLines1And2*NumbersPrScanLine);
	memcpy(pFontData+ScanLines1And2*NumbersPrScanLine,Data2,ScanLines1And2*NumbersPrScanLine);
	memcpy(pFontData+2*ScanLines1And2*NumbersPrScanLine,Data3,ScanLines3*NumbersPrScanLine);

	// set to actual font
	m_FontData[CourierBold13x18]=pFontData;
}

void CGraphics::InitCourier8x12()
{
	UINT8 Data1[]={135,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,247,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,247,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,247,247,235,177,255,235,255,255,247,247,235,255,247,247,235,255,255,177,247,
		247,235,177,255,255,255,247,247,235,255,247,255,255,255,255,255,255,255,247,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0, /*end of scanline 0 */
		127,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,247,235,245,247,223,231,247,251,239,255,255,255,255,255,255,227,247,227,227,251,193,243,193,227,227,255,255,255,255,255,227,225,231,129,225,131,128,128,
		225,136,193,225,140,131,156,152,227,129,227,129,193,128,136,136,136,136,136,128,227,255,227,235,255,239,255,159,255,249,255,243,255,159,247,251,159,231,255,255,255,255,255,255,255,255,255,255,255,255,255,255,249,247,207,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,247,247,255,255,255,255,255,255,255,255,255,255,255,255,255,255,247,255,255,190,255,247,225,221,227,231,255,255,255,227,255,247,247,247,231,251,255,192,255,255,231,231,255,159,159,159,247,231,231,231,231,231,247,192,255,128,128,128,128,193,193,193,193,131,152,227,
		227,227,227,227,255,226,136,136,136,136,136,159,227,239,251,247,198,221,235,255,255,239,251,247,221,239,247,247,219,201,198,239,251,247,198,221,255,255,239,251,247,221,251,159,221,0, /*end of scanline 1 */
		99,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,255,247,235,245,227,174,223,247,247,247,255,255,255,255,255,254,221,199,221,221,243,223,239,221,221,221,255,255,255,255,255,221,222,247,222,222,221,222,222,
		222,221,247,251,221,239,221,221,221,222,221,222,190,182,221,221,221,221,221,189,239,191,251,221,255,247,255,223,255,253,255,239,255,223,255,255,223,247,255,255,255,255,255,255,255,239,255,255,255,255,255,255,247,247,247,206,227,227,227,227,227,227,227,227,227,227,227,227,227,227,
		227,227,227,227,239,239,227,227,227,227,227,227,227,227,227,227,227,227,227,255,255,255,231,193,136,247,221,255,221,251,255,255,255,221,255,235,247,235,251,247,255,133,255,255,247,219,255,222,222,238,255,247,247,247,247,247,247,230,225,222,222,222,222,247,247,247,247,221,221,221,
		221,221,221,221,255,221,221,221,221,221,221,223,221,247,247,235,177,255,247,255,255,247,247,235,255,247,239,235,255,247,177,247,247,235,177,255,255,255,247,247,235,255,247,223,255,0, /*end of scanline 2 */
		99,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,255,247,235,192,221,221,223,247,247,247,201,247,255,255,255,253,221,247,253,253,243,223,223,253,221,221,255,255,251,255,239,253,178,247,222,191,222,223,223,
		191,221,247,251,219,239,201,205,190,222,190,222,191,247,221,221,221,235,221,251,239,223,251,255,255,255,255,223,255,253,255,239,255,223,255,255,223,247,255,255,255,255,255,255,255,239,255,255,255,255,255,255,247,247,247,182,227,227,227,227,227,227,227,227,227,227,227,227,227,227,
		227,227,227,227,255,255,227,227,227,227,227,227,227,227,227,227,227,227,227,255,247,247,219,221,221,247,175,255,178,227,255,255,255,166,255,247,247,251,247,255,255,133,255,255,247,219,255,221,221,221,247,247,247,247,247,247,247,231,222,223,223,223,223,247,247,247,247,222,205,190,
		190,190,190,190,190,186,221,221,221,221,221,193,221,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,203,255,255,255,255,255,255,247,255,255,255,255,255,255,223,255,0, /*end of scanline 3 */
		99,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,255,247,255,235,223,251,239,255,239,251,227,247,255,255,255,251,221,247,253,253,235,223,223,251,221,221,247,247,247,193,247,253,170,235,222,191,222,219,219,
		191,221,247,251,219,239,201,205,190,222,190,222,191,247,221,221,221,235,235,251,239,239,251,255,255,255,195,193,193,193,193,195,192,211,199,195,217,247,139,147,193,129,192,137,193,195,153,136,136,136,136,129,247,247,247,185,227,227,227,227,227,227,227,227,227,227,227,227,227,227,
		227,227,227,227,255,255,227,227,227,227,227,227,227,227,227,227,227,227,227,255,247,227,223,221,128,247,183,255,174,219,255,193,255,170,255,255,128,247,251,255,221,133,255,255,247,219,255,219,219,235,247,235,235,235,235,235,235,213,191,219,219,219,219,247,247,247,247,222,205,190,
		190,190,190,190,221,186,221,221,221,221,235,222,221,195,195,195,195,195,195,201,193,193,193,193,193,199,199,199,199,253,147,193,193,193,193,193,255,192,153,153,153,153,136,193,136,0 /*end of scanline 4 */
	};
	UINT8 Data2[]={227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,255,247,255,235,227,247,207,255,239,251,128,247,255,255,255,247,221,247,251,243,235,195,195,251,227,225,255,255,239,255,251,251,170,235,193,191,222,195,195,
		191,193,247,251,215,239,213,213,190,193,190,193,193,247,221,235,213,247,235,247,239,247,251,255,255,255,253,222,190,189,190,239,189,205,247,251,219,247,213,205,190,222,189,230,190,239,221,221,221,221,221,187,247,247,247,255,227,227,227,227,227,227,227,227,227,227,227,227,227,227,
		227,227,227,227,255,255,227,227,227,227,227,227,227,227,227,227,227,227,227,255,247,221,239,221,235,247,219,255,174,227,238,253,255,166,255,255,247,227,231,255,221,197,255,255,227,231,187,134,133,150,239,235,235,235,235,235,235,209,191,195,195,195,195,247,247,247,247,134,213,190,
		190,190,190,190,235,182,221,221,221,221,235,222,209,253,253,253,253,253,253,246,190,190,190,190,190,247,247,247,247,193,205,190,190,190,190,190,255,188,221,221,221,221,221,222,221,0, /*end of scanline 5 */
		227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,255,247,255,235,253,239,182,255,239,251,227,128,255,128,255,239,221,247,247,253,219,253,221,247,221,253,255,255,223,193,253,247,170,221,222,191,222,219,219,
		184,221,247,251,199,239,213,217,190,223,190,219,254,247,221,235,213,235,247,239,239,251,251,255,255,255,193,222,191,189,128,239,189,221,247,251,215,247,213,221,190,222,189,239,199,239,221,221,213,227,221,247,207,247,249,255,227,227,227,227,227,227,227,227,227,227,227,227,227,227,
		227,227,227,227,255,255,227,227,227,227,227,227,227,227,227,227,227,227,227,255,247,223,195,193,128,255,237,255,178,255,221,255,128,170,255,255,247,255,255,255,221,245,247,255,255,255,221,236,234,236,223,221,221,221,221,221,221,213,191,219,219,219,219,247,247,247,247,222,217,190,
		190,190,190,190,247,174,221,221,221,221,247,222,222,193,193,193,193,193,193,192,191,128,128,128,128,247,247,247,247,190,221,190,190,190,190,190,128,178,221,221,221,221,221,222,221,0, /*end of scanline 6 */
		227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,255,247,255,129,221,221,181,255,239,251,201,247,255,255,255,223,221,247,239,253,193,253,221,247,221,253,255,255,239,255,251,247,177,193,222,191,222,223,223,
		190,221,247,187,219,239,221,217,190,223,190,219,254,247,221,235,213,235,247,239,239,253,251,255,255,255,189,222,191,189,191,239,189,221,247,251,199,247,213,221,190,222,189,239,249,239,221,235,213,227,235,239,247,247,247,255,227,227,227,227,227,227,227,227,227,227,227,227,227,227,
		227,227,227,227,255,255,227,227,227,227,227,227,227,227,227,227,227,227,227,255,247,221,239,190,247,247,245,255,221,195,187,255,255,221,255,255,247,255,255,255,221,245,255,255,255,195,238,218,222,218,223,193,193,193,193,193,193,135,191,223,223,223,223,247,247,247,247,222,217,190,
		190,190,190,190,235,174,221,221,221,221,247,193,222,189,189,189,189,189,189,183,191,191,191,191,191,247,247,247,247,190,221,190,190,190,190,190,255,166,221,221,221,221,235,222,235,0, /*end of scanline 7 */
		227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,255,255,255,215,227,186,187,255,239,251,255,247,255,255,255,191,221,247,223,221,251,221,221,239,221,251,255,255,247,255,247,255,223,221,222,222,221,222,223,
		222,221,247,187,221,238,221,221,221,223,221,221,190,247,221,247,235,221,247,222,239,254,251,255,255,255,189,222,190,189,190,239,193,221,247,251,219,247,213,221,190,222,189,239,190,237,217,235,235,221,235,221,247,247,247,255,227,227,227,227,227,227,227,227,227,227,227,227,227,227,
		227,227,227,227,255,255,227,227,227,227,227,227,227,227,227,227,227,227,227,255,247,227,239,255,247,247,187,255,227,255,221,255,255,227,255,255,255,255,255,255,221,245,255,255,255,255,221,176,189,176,221,221,221,221,221,221,221,182,222,222,222,222,222,247,247,247,247,221,221,221,
		221,221,221,221,221,221,221,221,221,221,247,223,222,189,189,189,189,189,189,182,190,190,190,190,190,247,247,247,247,190,221,190,190,190,190,190,255,158,217,217,217,217,235,222,235,0, /*end of scanline 8 */
		227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,255,247,255,215,247,253,196,255,247,247,255,247,247,255,247,255,227,193,193,227,241,227,227,239,227,231,247,247,251,255,239,247,227,136,129,225,131,128,135,
		225,136,193,199,140,128,136,141,227,135,227,140,193,227,227,247,235,136,227,128,239,255,251,255,255,255,194,129,193,192,193,195,253,136,193,251,156,193,148,136,193,193,193,131,193,243,228,247,235,136,247,129,247,247,247,255,227,227,227,227,227,227,227,227,227,227,227,227,227,227,
		227,227,227,227,255,255,227,227,227,227,227,227,227,227,227,227,227,227,227,255,247,247,193,255,227,247,135,255,255,255,238,255,255,255,255,255,128,255,255,255,195,228,255,247,255,255,187,254,248,254,227,136,136,136,136,136,136,176,225,128,128,128,128,193,193,193,193,131,141,227,
		227,227,227,227,190,163,227,227,227,227,227,143,145,194,194,194,194,194,194,201,193,193,193,193,193,193,193,193,193,193,136,193,193,193,193,193,247,129,228,228,228,228,247,193,247,0 /*end of scanline 9 */
	};
	UINT8 Data3[]={227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,227,255,255,255,255,255,255,255,255,247,247,255,255,239,255,255,255,255,255,255,255,255,255,255,255,255,255,255,239,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,228,255,255,255,255,255,255,255,255,255,239,255,251,255,255,255,255,255,255,255,255,255,253,255,255,251,255,255,255,255,255,223,253,255,255,255,255,255,255,255,247,255,247,247,247,255,227,227,227,227,227,227,227,227,227,227,227,227,227,227,
		227,227,227,227,255,255,227,227,227,227,227,227,227,227,227,227,227,227,227,255,255,255,255,255,255,247,255,255,255,255,255,255,255,255,255,255,255,255,255,255,223,255,255,251,255,255,255,255,255,255,255,255,255,255,255,255,255,255,251,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,251,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,247,223,247,0, /*end of scanline 10 */
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,251,239,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,227,255,227,255,0,255,255,255,255,255,255,255,195,255,255,199,255,255,255,255,255,143,248,255,255,255,255,255,255,255,207,255,249,247,207,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,247,255,255,255,255,255,255,255,255,255,255,255,255,255,255,191,255,255,231,255,255,255,255,255,255,255,255,255,255,255,255,255,255,243,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,243,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,207,143,207,0 /*end of scanline 11 */
	};
	// following two variables must correspond to numbers of scanlines in each variable above, 1 and 2 must be the same
	int ScanLines1And2=5; // number of scanlines stored in each variable xxxxData1 and xxxxData2
	int ScanLines3=2; // number of scanlines stored in variable xxxxData3
	// following two variables can be found in output file written by the font data generator
	int TotalNumbers=3084; // total numbers defining font
	int NumbersPrScanLine=257; // numbers pr scanline
	
	UINT8* pFontData=(UINT8*)malloc(TotalNumbers);
	memcpy(pFontData,Data1,ScanLines1And2*NumbersPrScanLine);
	memcpy(pFontData+ScanLines1And2*NumbersPrScanLine,Data2,ScanLines1And2*NumbersPrScanLine);
	memcpy(pFontData+2*ScanLines1And2*NumbersPrScanLine,Data3,ScanLines3*NumbersPrScanLine);

	// set to actual font
	m_FontData[Courier8x12]=pFontData;
}


void CGraphics::InitTerminal12x16()
{
	UINT8 Data1[]={255,255,143,248,255,255,255,255,255,255,255,255,0,15,255,0,15,255,255,255,223,243,255,255,255,255,255,255,255,255,255,255,255,255,255,223,255,255,255,255,255,255,255,255,255,255,255,255,255,255,159,255,255,255,249,255,255,255,255,31,252,126,63,255,255,255,255,255,255,255,255,
		255,240,127,207,224,62,3,254,56,0,252,56,0,240,126,3,255,255,255,255,63,255,207,254,7,224,63,159,128,255,7,128,248,1,128,31,3,159,158,7,255,153,249,159,249,249,159,159,15,128,127,15,128,126,7,192,57,249,159,153,249,159,153,249,128,30,7,255,254,7,
		253,255,255,255,255,255,159,255,255,255,159,255,248,127,255,159,255,255,255,252,255,241,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,195,249,252,63,255,255,255,240,127,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,230,127,159,254,127,255,248,31,255,255,255,255,255,255,255,255,254,103,255,255,255,255,255,255,255,255,199,255,255,255,255,255,255,255,255,19,224,62,7,249,255,255,255,255,255,255,255,159,255,255,255,119,122,170,136,143,159,249,255,207,249,255,63,255,252,159,201,255,
		255,201,255,255,159,159,255,249,255,159,255,255,159,255,255,159,255,255,19,201,255,255,201,255,255,201,255,255,201,255,255,255,252,15,249,254,103,231,255,255,254,127,159,230,127,159,255,240,0,255,255,159,231,240,0,254,127,255,249,254,127,255,255,19,255,255,255,195,255,231,
		249,254,127,255,255,231,255,255,207,255,255,255,255,255,255,255,255,255,255,255,255,240,255,255,255,254,127,195,252,63,255,255,255,0, /*end of scanline 0 */
		255,254,115,224,63,255,255,255,255,255,255,255,0,15,255,0,15,255,255,255,207,241,255,223,255,255,255,253,254,103,224,30,7,255,255,143,253,255,223,255,255,255,255,255,255,255,255,255,255,255,159,255,255,153,249,255,255,248,255,31,249,255,159,255,255,255,255,255,255,255,255,
		254,192,31,143,192,28,1,252,56,0,248,56,0,224,60,1,255,255,255,254,63,255,199,252,3,192,31,159,128,126,3,128,120,1,128,30,1,159,158,7,255,153,241,159,248,241,143,158,7,128,62,7,128,60,3,192,57,249,159,153,249,159,153,249,128,30,7,191,254,7,
		248,255,255,248,255,255,159,255,255,255,159,255,240,127,255,159,255,255,255,252,255,241,255,255,255,255,255,255,255,255,255,255,255,231,255,255,255,255,255,255,255,255,255,255,131,249,252,31,255,255,255,224,63,255,254,127,159,255,255,63,249,255,255,249,255,255,243,255,255,249,
		255,63,230,127,111,252,255,255,248,31,159,255,255,63,249,255,63,255,254,103,255,255,255,255,255,255,255,255,131,252,255,207,252,255,207,255,254,71,192,28,3,249,255,255,255,255,255,255,255,159,255,255,255,221,213,85,34,47,159,249,255,159,240,255,159,255,252,159,201,255,
		255,201,255,159,159,159,255,249,255,159,255,255,159,255,255,159,255,254,71,201,255,255,201,255,255,201,255,255,201,249,249,255,252,7,240,254,103,243,255,159,252,255,15,230,127,159,255,240,0,255,255,159,243,240,0,252,255,255,240,255,63,255,254,71,255,255,255,195,255,207,
		240,255,63,255,255,207,255,255,159,255,255,255,255,255,255,224,30,7,255,255,255,224,127,255,255,252,127,249,255,159,255,255,255,0, /*end of scanline 1 */
		255,253,253,192,31,255,255,255,143,253,255,255,0,15,255,0,15,255,248,127,199,244,255,223,239,255,247,248,254,103,201,60,243,255,255,7,248,255,223,255,255,255,255,255,255,255,255,255,255,255,15,230,127,153,224,127,254,242,127,31,241,255,143,255,255,255,255,255,255,255,255,
		252,207,158,15,143,136,248,248,57,255,241,255,249,199,24,248,255,255,255,252,127,255,227,248,241,207,159,15,158,60,113,158,57,255,159,252,121,159,159,159,255,153,227,159,248,241,143,156,99,159,28,99,159,24,241,249,249,249,159,153,249,207,60,243,255,62,127,159,255,231,
		240,127,255,248,255,255,159,255,255,255,159,255,227,255,255,159,255,159,254,124,255,249,255,255,255,255,255,255,255,255,255,255,255,231,255,255,255,255,255,255,255,255,255,255,31,249,255,143,227,159,255,199,30,103,252,255,15,230,127,159,246,255,255,240,254,103,249,254,79,240,
		255,159,255,255,159,249,255,255,240,255,15,230,127,159,240,255,159,230,127,255,230,127,255,248,126,9,255,255,147,249,255,159,249,255,159,241,63,255,255,152,241,255,254,3,255,252,255,207,255,255,255,255,255,119,122,170,136,143,159,249,255,63,230,127,207,224,60,159,201,255,
		255,201,255,159,207,63,255,249,255,159,255,255,159,255,255,159,241,63,255,201,255,255,201,255,255,201,255,255,201,249,9,240,252,227,230,127,255,249,255,159,249,254,103,255,255,159,255,240,0,255,255,159,249,240,0,249,255,255,230,127,159,226,127,255,255,252,63,231,255,159,
		230,127,159,254,127,159,248,127,63,255,255,255,255,248,255,201,60,243,255,255,255,230,127,255,255,254,127,227,254,63,255,255,255,0, /*end of scanline 2 */
		255,253,253,192,30,115,253,255,7,253,255,255,0,15,255,0,15,225,243,63,195,246,125,141,231,255,231,240,126,103,153,60,255,255,254,3,240,127,223,255,255,255,255,255,255,255,255,255,255,255,15,230,127,153,192,60,124,230,127,159,243,255,207,201,63,255,255,255,255,255,255,
		248,159,142,15,159,201,252,241,57,255,227,255,249,207,153,252,255,255,255,248,255,255,241,249,249,152,79,15,159,60,249,159,57,255,159,252,255,159,159,159,255,153,199,159,248,97,135,156,243,159,156,243,159,153,249,249,249,249,207,57,249,207,60,243,255,62,127,143,255,231,
		226,63,255,248,255,255,159,255,255,255,159,255,231,255,255,159,255,159,254,124,255,249,255,255,255,255,255,255,255,255,255,255,255,231,255,255,255,255,255,255,255,255,255,255,63,249,255,207,201,63,255,207,158,103,249,254,103,230,127,207,249,255,255,230,126,103,252,254,79,230,
		127,207,249,255,255,128,31,255,240,254,103,230,127,207,230,127,207,230,126,7,230,127,255,240,60,3,255,255,159,243,255,63,243,255,63,228,121,249,192,25,249,249,252,1,0,24,255,143,255,255,255,255,255,221,213,85,34,47,159,249,255,159,249,255,159,192,28,159,201,255,
		255,201,255,7,207,63,255,249,255,159,255,255,159,255,255,159,228,127,159,201,255,255,201,255,255,201,255,255,201,252,3,224,124,243,128,24,1,128,31,159,224,126,7,224,127,159,255,240,0,255,255,159,224,112,0,224,127,255,224,126,7,200,254,7,255,252,63,224,121,249,
		159,153,249,252,249,249,248,127,255,255,255,255,255,254,127,153,60,255,255,255,255,230,127,255,255,254,127,249,252,255,255,255,255,0, /*end of scanline 3 */
		255,251,38,141,140,33,248,255,7,248,255,255,0,15,159,6,15,249,231,159,211,243,62,3,227,255,199,224,62,103,153,60,255,255,255,223,224,63,223,254,255,191,255,255,119,255,255,255,255,255,15,230,124,0,201,252,120,230,127,159,227,255,199,201,63,159,255,255,255,255,255,
		241,159,15,207,159,143,252,227,57,255,199,255,243,207,153,252,241,255,31,241,255,255,248,249,241,144,79,15,159,57,255,159,153,255,159,249,255,159,159,159,255,153,143,159,248,97,147,153,249,159,153,249,159,153,255,249,249,249,207,57,249,230,126,103,254,126,127,199,255,231,
		199,31,255,249,255,255,159,255,255,255,159,255,231,255,255,159,255,255,255,252,255,249,255,255,255,255,255,255,255,255,255,255,255,231,255,255,255,255,255,255,255,255,255,255,63,249,255,207,156,127,159,159,159,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,249,255,159,128,31,255,224,255,255,255,255,255,255,255,255,255,252,3,255,255,255,243,56,225,207,63,159,255,255,255,255,255,255,255,248,249,128,25,249,249,248,248,0,28,243,207,63,159,255,255,255,119,122,170,136,143,159,249,255,159,249,255,159,143,140,159,201,255,
		255,201,254,3,230,127,255,249,255,159,255,255,159,255,255,159,255,255,159,201,255,255,201,255,255,201,255,255,201,248,241,231,124,249,128,24,1,128,31,159,224,126,7,224,127,159,255,240,0,255,255,159,224,112,0,192,63,15,192,60,3,255,252,3,159,62,127,224,57,249,
		159,153,249,249,252,243,255,255,255,255,255,159,255,252,243,153,60,255,249,255,255,224,127,255,255,254,127,195,252,31,240,255,255,0, /*end of scanline 4 */
		255,251,38,141,140,1,240,127,7,240,127,255,0,15,111,9,14,21,231,159,211,241,190,115,225,255,135,253,254,103,153,62,7,255,255,223,253,255,223,254,127,63,255,254,115,253,248,0,255,255,15,230,127,51,201,252,113,228,255,63,227,255,199,224,127,159,255,255,255,255,255,
		227,158,79,207,255,31,248,199,56,3,207,255,243,199,24,248,241,255,31,227,252,1,252,127,227,146,78,103,158,57,255,159,153,255,159,249,255,159,159,159,255,153,31,159,249,9,147,153,249,159,153,249,159,152,255,249,249,249,207,57,249,240,254,103,252,254,127,227,255,231,
		143,143,255,249,254,3,144,126,7,224,158,7,231,254,1,144,255,31,252,124,231,249,250,103,192,126,7,128,126,1,200,60,15,128,249,249,159,153,153,159,60,243,128,63,63,249,255,207,255,255,15,159,249,249,224,126,3,224,62,3,224,62,7,224,126,7,224,127,31,241,
		255,31,240,255,15,159,255,255,228,254,7,224,126,7,159,153,249,207,56,241,159,158,9,243,249,201,230,127,159,224,63,31,224,121,249,192,120,121,159,152,241,249,249,12,255,156,231,206,127,159,231,57,207,221,213,85,34,47,159,249,255,15,240,255,15,152,204,159,201,255,
		255,201,252,19,230,127,255,249,255,159,255,255,159,255,255,159,224,63,15,201,255,255,201,255,255,201,255,255,201,249,249,231,252,249,159,249,255,159,255,159,249,255,159,249,255,159,255,240,0,255,255,159,249,240,0,143,30,103,143,24,241,224,120,241,159,62,7,231,57,249,
		159,153,249,207,62,103,255,255,255,255,255,159,255,254,103,153,62,7,249,255,255,240,255,255,255,255,255,255,255,255,240,255,255,0, /*end of scanline 5 */
		255,251,254,128,12,1,224,60,137,224,63,159,6,14,247,16,140,205,231,159,215,244,188,249,224,255,7,253,254,103,201,60,243,255,255,223,253,255,223,254,62,63,255,252,113,248,252,1,255,255,15,255,255,51,192,127,227,241,255,255,227,255,199,240,255,159,255,255,255,255,255,
		199,156,207,207,254,63,1,143,60,1,128,63,231,224,60,0,241,255,31,199,252,1,254,63,199,146,78,103,128,121,255,159,152,7,128,121,193,128,31,159,255,152,63,159,249,9,153,153,249,159,25,249,159,28,7,249,249,249,230,121,249,249,255,15,249,254,127,241,255,231,
		159,207,255,252,252,1,128,60,3,192,28,3,128,252,1,128,127,31,252,124,199,249,248,3,192,60,3,128,60,1,192,24,7,128,249,249,159,153,153,142,60,243,128,126,63,255,255,199,255,254,103,159,249,249,192,60,1,192,28,1,192,28,3,192,60,3,192,63,31,241,
		255,31,240,255,15,159,252,35,228,60,3,192,60,3,159,153,249,207,57,249,159,156,3,243,249,201,240,255,159,192,31,31,192,57,249,192,56,57,128,28,3,241,249,36,255,152,79,132,255,159,206,124,231,119,122,170,136,143,159,249,255,15,240,255,15,146,64,159,201,240,
		31,9,252,159,240,255,255,249,255,159,255,255,159,255,255,159,192,31,15,200,12,0,8,0,0,200,0,0,8,9,249,227,248,121,159,249,255,159,255,159,249,255,159,249,255,159,255,240,0,255,255,255,249,240,0,159,156,243,159,153,249,192,57,249,159,62,3,231,57,249,
		159,153,249,207,63,15,255,255,255,224,126,7,224,120,207,201,60,243,255,254,207,255,254,247,253,255,255,255,255,255,240,255,255,0 /*end of scanline 6 */
	};
	UINT8 Data2[]={255,251,254,128,12,1,192,24,0,192,31,15,15,14,247,16,137,231,243,63,223,246,56,248,224,126,7,253,254,103,225,60,243,255,255,223,253,255,223,192,28,1,255,248,0,240,126,3,255,255,159,255,254,103,224,63,199,225,255,255,227,255,199,192,60,3,255,252,3,255,255,
		143,153,207,207,252,127,3,159,63,248,128,31,231,192,30,0,255,255,255,199,255,255,254,63,143,146,78,103,128,57,255,159,152,7,128,121,193,128,31,159,255,152,63,159,249,153,153,153,249,128,57,249,128,62,3,249,249,249,230,121,153,249,255,15,249,254,127,248,255,231,
		255,255,255,255,255,249,143,24,249,142,24,249,128,248,249,142,63,159,254,124,143,249,248,1,207,24,241,159,24,249,199,153,255,231,249,249,207,57,153,196,126,103,252,252,127,249,255,227,255,252,243,159,249,249,143,159,249,255,159,249,255,152,241,143,152,249,143,159,159,249,
		255,159,230,126,103,128,121,153,204,56,241,143,24,241,159,153,249,230,121,249,159,152,225,224,249,153,249,254,7,255,159,159,143,25,249,207,25,25,192,30,7,227,249,12,255,159,159,249,255,15,156,254,115,221,213,85,34,47,159,1,254,103,230,126,103,147,192,159,201,240,
		31,9,252,159,240,240,31,248,0,0,0,15,128,0,0,0,255,158,103,200,12,0,8,0,0,200,0,0,8,9,249,241,248,121,128,120,7,128,127,255,249,255,159,249,240,31,248,0,0,255,255,159,249,240,0,159,156,243,159,153,249,143,25,249,159,62,115,231,57,249,
		159,153,249,230,127,159,255,255,255,224,126,7,224,127,159,225,60,243,192,63,15,255,252,99,248,255,255,255,255,255,240,255,255,0, /*end of scanline 7 */
		255,251,118,136,142,3,224,56,0,192,31,15,15,14,247,16,137,231,248,127,223,199,60,249,224,255,7,253,254,103,249,62,7,255,255,223,253,255,223,254,62,63,191,252,113,224,63,7,255,255,159,255,254,103,249,63,143,193,255,255,227,255,199,240,252,3,255,252,3,255,255,
		31,147,207,207,248,255,249,128,15,252,143,143,207,143,143,249,255,255,255,227,255,255,252,127,159,146,76,243,159,25,255,159,153,255,159,249,249,159,159,159,255,153,31,159,249,153,156,153,249,128,121,249,128,127,241,249,249,249,230,121,153,240,255,159,243,254,127,252,127,231,
		255,255,255,255,254,1,159,153,255,159,152,1,231,249,249,159,63,159,254,124,31,249,249,153,207,153,249,159,153,249,207,248,15,231,249,249,207,57,153,224,254,103,249,254,63,249,255,199,255,249,249,159,249,249,128,30,1,224,30,1,224,25,255,128,24,1,128,31,159,249,
		255,159,230,126,103,128,127,153,204,249,249,159,153,249,159,153,249,230,121,249,159,153,201,243,249,153,240,255,159,224,31,159,159,153,249,207,153,137,255,255,255,199,249,36,255,159,35,242,159,15,206,124,231,119,122,170,136,143,159,1,254,103,230,126,103,146,79,159,201,255,
		159,249,252,159,249,240,31,248,0,0,0,15,128,0,0,0,224,30,103,207,252,255,255,255,255,207,255,255,255,249,249,248,252,249,128,120,7,128,127,255,249,255,159,249,240,31,248,0,0,0,15,159,249,255,255,159,156,7,159,153,249,159,153,249,159,62,115,231,57,249,
		159,153,249,230,127,159,255,255,255,255,255,159,255,255,41,249,62,7,192,63,255,255,252,99,248,255,255,255,255,255,240,255,255,0, /*end of scanline 8 */
		255,251,142,135,15,7,240,120,0,192,31,159,6,14,247,16,137,231,252,255,31,135,190,115,225,255,135,253,254,103,249,63,243,255,255,223,253,255,223,254,127,63,191,254,115,192,31,143,255,255,159,255,248,3,249,63,31,204,159,255,227,255,199,224,127,159,255,255,255,255,254,
		63,135,207,207,241,255,252,128,15,252,159,207,207,159,207,241,255,255,255,241,252,1,248,255,159,144,28,3,159,153,255,159,153,255,159,249,249,159,159,159,159,153,143,159,249,249,156,153,249,159,249,201,152,255,249,249,249,249,240,249,9,230,127,159,231,254,127,254,63,231,
		255,255,255,255,252,1,159,153,255,159,152,3,231,248,241,159,63,159,254,124,31,249,249,153,207,153,249,159,153,249,207,252,7,231,249,249,230,121,153,241,255,15,243,255,63,249,255,207,255,249,249,207,153,249,128,28,1,192,28,1,192,25,255,128,24,1,128,31,159,249,
		255,159,207,60,243,159,252,3,192,249,249,159,153,249,159,153,249,240,249,249,159,153,153,243,249,57,230,127,159,192,31,159,159,153,249,207,153,193,192,60,3,143,153,36,255,158,121,228,159,15,231,57,207,221,213,85,34,47,159,249,252,243,207,60,243,152,192,159,201,240,
		159,1,252,19,192,63,159,255,255,255,249,255,159,255,255,159,192,28,243,192,12,128,0,0,128,200,0,0,8,8,241,240,124,249,159,249,255,159,255,255,249,255,159,249,255,255,249,240,0,0,15,159,249,255,255,159,156,227,159,153,249,159,153,249,142,62,3,224,57,249,
		159,153,249,240,255,159,255,255,255,255,255,159,224,126,73,249,63,243,255,255,255,255,254,247,253,255,255,255,255,255,240,255,255,0, /*end of scanline 9 */
		255,253,253,192,31,143,248,252,217,229,63,255,0,15,111,9,12,207,252,254,31,135,190,3,227,255,199,224,62,103,249,63,243,255,254,3,253,254,3,254,255,191,191,255,119,128,15,223,255,255,255,255,252,207,192,62,63,204,63,255,243,255,207,201,63,159,255,255,255,255,252,
		127,143,207,207,227,249,252,255,57,252,159,207,159,159,207,227,241,255,31,248,252,1,241,255,159,152,60,3,159,156,249,159,57,255,159,252,249,159,159,159,159,153,199,159,249,249,158,28,243,159,252,195,156,121,249,249,249,249,240,248,97,207,63,159,207,254,127,255,31,231,
		255,255,255,255,249,249,159,153,255,159,153,255,231,252,1,159,63,159,254,124,143,249,249,153,207,153,249,143,24,241,207,255,231,231,249,249,230,121,9,224,255,15,231,255,63,249,255,207,255,248,1,199,25,249,159,249,249,159,153,249,159,153,255,159,249,255,159,255,159,249,
		255,159,192,60,3,159,249,159,128,249,249,159,153,249,159,153,249,240,249,249,159,153,57,231,249,57,207,63,159,159,159,159,159,153,249,207,153,225,192,60,3,159,152,248,255,252,243,204,31,15,255,255,255,119,122,170,136,143,159,249,252,3,192,60,3,143,128,159,201,240,
		159,1,254,3,249,255,159,255,255,255,249,255,159,255,255,159,159,156,3,192,12,128,0,0,128,200,0,0,8,12,3,230,124,243,159,249,255,159,255,255,249,255,159,249,255,255,249,240,0,0,15,159,249,255,255,159,156,243,159,153,249,159,153,249,128,62,7,224,121,249,
		159,153,249,240,255,159,255,255,255,255,255,255,224,124,193,249,63,243,249,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0, /*end of scanline 10 */
		255,253,253,192,31,223,253,255,143,248,255,255,0,15,159,6,14,31,240,62,31,206,61,141,231,255,231,240,127,255,249,60,243,224,31,7,253,255,7,255,255,255,191,255,255,255,255,255,255,255,255,255,252,207,224,124,113,206,127,255,241,255,143,201,63,159,241,255,255,241,248,
		255,207,159,207,199,248,248,255,56,248,143,143,159,143,143,199,241,255,31,252,127,255,227,255,255,143,249,249,159,28,113,158,57,255,159,252,121,159,159,159,143,57,227,159,249,249,159,28,99,159,252,99,158,56,241,249,252,243,240,248,241,207,63,159,207,254,127,255,143,231,
		255,255,255,255,249,249,159,24,249,143,152,255,231,254,9,159,63,159,254,124,199,249,249,153,207,152,241,128,60,1,207,255,231,231,248,241,240,252,3,196,127,159,207,255,63,249,255,207,255,248,1,224,56,241,143,249,249,159,153,249,159,152,241,159,249,255,159,255,159,249,
		255,159,128,24,1,159,249,159,156,248,241,143,24,241,143,24,241,249,248,241,143,24,113,192,24,113,255,255,159,159,159,159,143,24,241,207,153,241,255,255,255,143,28,1,255,255,231,255,159,15,255,255,255,221,213,85,34,47,159,249,248,1,128,24,1,192,28,159,201,252,
		159,255,255,7,192,63,159,255,255,255,249,255,159,255,255,159,159,152,1,255,252,159,255,252,159,201,255,255,201,249,9,230,124,227,159,249,255,159,255,255,249,255,159,249,255,255,249,240,0,0,15,159,249,255,255,143,28,243,143,24,241,143,24,241,129,62,127,231,252,243,
		207,60,243,249,255,159,255,255,255,255,254,7,255,255,249,249,60,243,249,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0, /*end of scanline 11 */
		255,254,115,224,63,255,255,255,7,240,127,255,0,15,255,0,15,255,252,255,63,252,63,223,239,255,247,248,254,103,249,62,7,224,31,143,253,255,143,255,255,255,191,255,255,255,255,255,255,255,159,255,252,207,249,248,241,196,63,255,249,255,159,255,255,255,241,255,255,241,249,
		255,192,30,1,128,12,1,255,60,1,192,31,63,192,30,15,241,255,31,254,63,255,199,255,159,192,57,249,128,62,3,128,120,1,159,254,1,159,158,7,192,57,241,128,25,249,159,30,7,159,254,1,159,28,3,249,252,3,249,248,241,159,159,159,128,30,7,255,206,7,
		255,255,255,255,248,1,128,60,3,192,28,3,231,255,249,159,62,7,254,124,227,224,121,153,207,156,3,144,126,9,207,248,7,224,124,1,240,252,99,142,63,159,128,63,31,249,255,143,255,255,255,240,124,1,192,56,1,128,24,1,128,28,3,192,60,3,192,62,7,224,
		126,7,159,153,249,128,25,153,156,28,3,192,60,3,192,28,1,249,252,3,192,28,3,196,60,3,255,252,159,128,30,7,192,60,1,207,153,249,255,255,255,192,62,3,255,255,225,255,159,159,255,255,255,119,122,170,136,143,159,249,249,249,159,153,249,224,60,159,201,252,
		159,255,255,159,249,255,159,255,255,255,249,255,159,255,255,159,128,25,249,255,252,159,255,252,159,201,255,255,201,249,249,224,124,7,128,24,1,128,31,255,224,126,7,224,127,255,249,240,0,0,15,159,224,127,255,192,60,227,192,60,3,192,60,3,159,252,63,195,252,3,
		192,60,3,249,255,159,255,255,255,255,254,7,255,255,249,249,62,7,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0, /*end of scanline 12 */
		255,255,143,248,255,255,255,255,255,255,255,255,0,15,255,0,15,255,252,255,255,252,63,223,255,255,255,253,254,103,249,63,255,224,31,223,253,255,223,255,255,255,128,31,255,255,255,255,255,255,159,255,255,255,249,249,241,225,159,255,252,126,63,255,255,255,241,255,255,241,255,
		255,240,126,1,128,14,3,255,62,3,224,63,63,224,62,31,255,255,159,255,63,255,207,255,159,240,57,249,128,127,7,128,248,1,159,255,1,159,158,7,224,121,249,128,25,249,159,159,15,159,255,9,159,158,7,249,254,7,249,249,249,159,159,159,128,30,7,255,254,7,
		255,255,255,255,252,1,128,126,7,224,30,7,231,255,241,159,62,7,230,124,243,224,121,153,207,158,7,159,255,249,207,252,15,240,126,9,249,254,247,159,63,63,128,63,131,249,252,31,255,255,255,249,254,9,224,124,1,192,28,1,192,30,7,224,126,7,224,126,7,224,
		126,7,159,153,249,128,28,67,156,30,7,224,126,7,224,158,9,243,254,7,224,153,7,255,249,7,255,252,31,192,30,7,224,126,9,207,153,249,255,255,255,224,127,255,255,255,255,255,255,159,255,255,255,221,213,85,34,47,159,249,249,249,159,153,249,255,252,159,201,252,
		159,255,255,159,249,255,159,255,255,255,249,255,159,255,255,159,192,25,249,255,252,159,255,252,159,201,255,255,201,255,255,240,252,15,128,24,1,128,31,255,224,126,7,224,127,255,249,240,0,0,15,159,224,127,255,224,124,7,224,126,7,224,126,7,159,252,63,195,254,7,
		224,126,7,243,255,159,255,255,255,255,255,255,255,255,255,249,63,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0 /*end of scanline 13 */
	};
	UINT8 Data3[]={255,255,255,255,255,255,255,255,255,255,255,255,0,15,255,0,15,255,255,255,255,254,127,255,255,255,255,255,255,255,255,255,255,255,254,3,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,249,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,159,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,248,0,255,255,255,255,255,255,255,255,255,255,252,3,255,255,255,224,127,255,255,255,255,255,255,255,159,255,249,255,255,255,255,255,255,255,255,255,255,255,63,255,255,195,255,252,63,255,255,255,225,255,255,255,255,255,255,255,255,255,255,159,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,243,255,255,255,255,255,255,255,255,255,254,63,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,119,122,170,136,143,159,249,255,255,255,255,255,255,252,159,201,252,
		159,255,255,255,255,255,159,255,255,255,249,255,159,255,255,159,255,255,255,255,252,159,255,252,159,201,255,255,201,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,249,240,0,0,15,255,255,255,255,255,252,255,255,255,255,255,255,255,159,255,255,255,255,255,
		255,255,255,243,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0, /*end of scanline 14 */
		255,255,255,255,255,255,255,255,255,255,255,255,0,15,255,0,15,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,243,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,63,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,248,0,255,255,255,255,255,255,255,255,255,255,252,7,255,255,255,240,255,255,255,255,255,255,255,255,159,255,249,255,255,255,255,255,255,255,255,255,255,254,127,255,255,255,255,255,255,255,255,255,225,255,255,255,255,255,255,255,255,255,254,31,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,231,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,221,213,85,34,47,159,249,255,255,255,255,255,255,252,159,201,252,
		159,255,255,255,255,255,159,255,255,255,249,255,159,255,255,159,255,255,255,255,252,159,255,252,159,201,255,255,201,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,249,240,0,0,15,255,255,255,255,255,252,255,255,255,255,255,255,255,159,255,255,255,255,255,
		255,255,255,231,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0, /*end of scanline 15 */
	};
	// following two variables must correspond to numbers of scanlines in each variable above, 1 and 2 must be the same
	int ScanLines1And2=7; // number of scanlines stored in each variable xxxxData1 and xxxxData2
	int ScanLines3=2; // number of scanlines stored in variable xxxxData3
	// following two variables can be found in output file written by the font data generator
	int TotalNumbers=6160; // total numbers defining font
	int NumbersPrScanLine=385; // numbers pr scanline
	
	UINT8* pFontData=(UINT8*)malloc(TotalNumbers);
	memcpy(pFontData,Data1,ScanLines1And2*NumbersPrScanLine);
	memcpy(pFontData+ScanLines1And2*NumbersPrScanLine,Data2,ScanLines1And2*NumbersPrScanLine);
	memcpy(pFontData+2*ScanLines1And2*NumbersPrScanLine,Data3,ScanLines3*NumbersPrScanLine);
	
	// set to actual font
	m_FontData[Terminal12x16]=pFontData;
}

} // end ipl namespace
