
/*
    GraadighetMapping er en implementasjon av rapportens Algortime 4
    Denne har til hensikt  avdekke eksekveringstider. Annenrangs
    mappinger er ikke inkludert, da disse ikke har vesentlig beydning
    for tiskompleksiteten. Klassen benytter seg av klassene
    QuickSort og ElementPar. De forskjellige  matriseinputene (input)
    er de samme som brukt i rapporten.

	Forfatter: Per Gunnar Brdsen
*/

import java.util.ArrayList;

public class GraadighetMapping
{

	/*
	// 5x5
	int[][] input = { {2,9,6,2,2},
					  {7,4,0,3,9},
					  {1,0,6,3,5},
					  {1,6,3,7,5},
					  {9,4,6,8,3} };
	*/

	/*
	// 6x7
	int[][] input= { {4,4,5,2,9,2,3},
				 	 {3,6,9,9,7,5,2},
			         {3,1,9,2,9,4,0},
			         {4,0,3,6,0,5,1},
			         {4,0,3,6,0,5,1},
			         {4,0,3,6,0,5,1} };

	// 6x8
	int[][] input= { {4,4,5,2,9,2,3,1},
				 	 {3,6,9,9,7,5,2,2},
			         {3,1,9,2,9,4,0,0},
			         {4,0,3,6,0,5,1,1},
			         {4,0,3,6,0,5,1,0},
			         {4,0,3,6,0,5,1,2} };

	// 6x10
	int[][] input = { {4,4,5,2,9,2,3,2,3,1},
			    	  {3,4,6,9,9,1,2,7,5,2},
			    	  {3,1,1,3,4,9,2,9,4,0},
			    	  {4,0,2,1,1,3,6,0,5,1},
			     	  {2,4,0,0,0,3,6,0,5,1},
			     	  {4,0,5,1,7,3,6,0,5,1} };

	*/
	// 7x7
	int[][] input = { {4,4,5,2,9,2,3},
			    	  {3,4,6,9,9,1,2},
			    	  {3,1,1,3,4,9,2},
			    	  {4,0,2,1,1,3,6},
			     	  {2,4,0,0,0,3,6},
			     	  {3,4,6,9,9,1,2},
			     	  {4,0,5,1,7,3,6} };
	//*/

	//Konstruktor og utskrift til skjerm
	public GraadighetMapping()
	{
		long before = System.currentTimeMillis();
		ArrayList s = sorter(input);
		System.out.println("\nINPUT:");
		skrivUtMatrise(input);
		System.out.println("\nSORTERT MATRISE:");
		skrivUtSortertMatrise(s);
		int[][] output = new int[input.length][input[0].length];
		iterer(s, output);
		System.out.println("\nOUTPUT:");
		skrivUtMatrise(output);
		long after = System.currentTimeMillis();
		System.out.println("\nTid (ms) = " + (after - before) + "\n");
	}

	//Sorterer matrisens rader etter stigende verdier
	private ArrayList sorter( int[][] m )
	{
		QuickSort qs = new QuickSort();
		ArrayList s = new ArrayList();
		ArrayList a = new ArrayList();
		for (int r=0;r<m.length;r++)
		{
			a.clear();
			for (int k=0;k<m[0].length;k++)
			{
				ElementPar ep = new ElementPar( m[r][k], k+1 );
				a.add( ep );
			}
			ArrayList sortert = qs.sort( a );
			ElementPar forrige = null;
			for (int k=0;k<sortert.size();k++)
			{
				ElementPar ep = (ElementPar) sortert.get(k);
				ep.f = forrige;
				if ( ep.f != null) { ep.f.n = ep; }
				if ( forrige == null )
				{
					ElementPar kvasiPar = new ElementPar(s.size()+1, -1);
					kvasiPar.n = ep;
					s.add( kvasiPar );
				}
				forrige = ep;
			}
		}
		return s;
	}

	//Itererende metode som velger beste mappinger i matrisen
	private void iterer( ArrayList s, int[][] output )
	{
		while ( s.size() > 0 )
		{
			int r=0, k=0, min=Integer.MAX_VALUE;
			for (int j=0;j<s.size();j++)
			{
				int temp = frontVerdi( s,j );
				if ( temp < min )
				{
					min = temp;
					k = frontKolonne( s,j );
					r = j;
				}
			}
			oppdaterPekere( s,k );
			int radPos = ((ElementPar) s.get(r)).v;
			output[radPos-1][k-1] = 1;
			s.remove(r);
		}
	}

	//Returnerer verdi til frste par i den aktuelle raden i s
	private int frontVerdi( ArrayList s, int j )
	{
		ElementPar kvasiPar = (ElementPar) s.get(j);
		return kvasiPar.n.v;
	}

	//Returnerer kolonne til frste par i den aktuelle raden i s
	private int frontKolonne( ArrayList s, int j )
	{
		ElementPar kvasiPar = (ElementPar) s.get(j);
		return kvasiPar.n.p;
	}

	//Oppdaterer pekere i matrisen nr elementer velges
	private void oppdaterPekere( ArrayList s, int k )
	{
		for (int i=0;i<s.size();i++)
		{
			ElementPar ep = (ElementPar) s.get(i);
			while ( ep.n != null )
			{
				if ( ep.n.p == k )
				{
					if (ep.n.n != null)
					{
						ep.n = ep.n.n;
						ep.n.f = ep;
					}
					else
					{
						ep.n = null;
					}
				}
				else
				{
					ep = ep.n;
				}
			}
		}
	}

	//Oppstartsmetode
	public static void main( String[] argumenter ) { new GraadighetMapping(); }

	//Skriver ut radsortert matrise av (verdi/kolonne)-par
	private void skrivUtSortertMatrise( ArrayList s )
	{
		for (int i=0;i<s.size();i++)
		{
			ElementPar ep = (ElementPar) s.get(i);
			System.out.println();
			while (ep.n != null)
			{
				System.out.print(" " + ep.n.v + "/" + ep.n.p);
				ep = ep.n;
			}
		}
		System.out.println();
	}

	//Skriver ut ferdig mappet matrise
	private void skrivUtMatrise(int[][] m)
	{
		for (int r=0;r<m.length;r++)
		{
			System.out.println();
			for (int k=0;k<m[0].length;k++)
			{
				System.out.print(" " + m[r][k]);
			}
		}
		System.out.println();
	}

}